!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine coupling2
          use types
          use Scalevar, only: doScalevar, maxscalevar
      implicit none
c--- this routine calculates alpha-s using the now-determined
c--- value of nflav and makes CKM matrix diagonal if necessary
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/masses.f'
c      include 'src/Inc/ewcouple.f'
      include 'src/Inc/qcdcouple.f'
      include 'src/Inc/scale.f'
      include 'src/Inc/verbose.f'
      include 'src/Inc/nlooprun.f'
c      include 'src/Inc/kprocess.f'
c      include 'src/Inc/ewinput.f'
      include 'src/Inc/nflav.f'
      include 'src/Inc/b0.f'
      include 'src/Inc/dynamicscale.f'
      include 'src/Inc/stopscales.f'
      include 'src/Inc/fourthgen.f'
      include 'src/Inc/couple.f'
      include 'src/Inc/kpart.f'
      include 'src/Inc/nproc.f'
      include 'src/Inc/taucut.f'
      include 'src/Inc/scalevar.f'
      include 'src/Inc/mpicommon.f'
      real(dp):: alphas,cmass,bmass
      real(dp):: Vud,Vus,Vub,Vcd,Vcs,Vcb
      common/cabib/Vud,Vus,Vub,
     &             Vcd,Vcs,Vcb
      common/qmass/cmass,bmass
c      common/em/aemmz
c      common/mykpart/mykpart
      logical, save :: first = .true.

c--- set up the beta-function
      b0=(xn*11._dp-2._dp*nflav)/6._dp

c--- initialize the pdf set
      nlooprun=0
      call pdfwrap      

      musq=scale**2
c--- set up masses used in running of alphas (alfamz.f)
      cmass=sqrt(mcsq)
      if (fourthgen) then
        continue ! normal b mass already set in chooser.f
      else
        bmass=sqrt(mbsq) ! use the mass specified in the input file
      endif
      
      if (nflav < 5) then
        bmass=1001._dp
      endif
      if (nflav < 4) then
        cmass=1000._dp
      endif
 
c--- set the number of loops to use in the running of alpha_s
c--- if it hasn't been set by pdfwrap already
      if (nlooprun == 0) then
        if (kpart==klord) then
          nlooprun=1
        else
          nlooprun=2
        endif
      endif

c--- initialize alpha_s
      as=alphas(abs(scale),amz,nlooprun)

      ason2pi=as/twopi
      ason4pi=as/fourpi
      gsq=fourpi*as

c--- if we're doing W + jets, automatically make the CKM matrix
c--- diagonal since we're not interested in these small effects   
      if ((nproc == 11) .or. (nproc == 16) .or.
     &    (nproc == 22) .or. (nproc == 27)) then
        if ( ((nproc == 11) .or. (nproc == 16))
     &        .and. usescet .and. abovecut ) then
! for W+jet process in above-cut SCET calculation, leave as-is
          continue
        else
          Vud=1._dp
          Vus=0._dp
          Vub=0._dp
          Vcd=0._dp
          Vcs=1._dp
          Vcb=0._dp
        endif
      endif

      if (verbose .and. first .and. rank == 0) then
      write(6,*)
      write(6,*) '***************** CKM mixing matrix ****************'
      write(6,*) '*                                                  *'
      write(6,47) Vud,Vus,Vub
      write(6,48) Vcd,Vcs,Vcb
      write(6,*) '****************************************************'
      if ((nproc == 11) .or. (nproc == 16) .or.
     &    (nproc == 22) .or. (nproc == 27)) then
        if ( ((nproc == 11) .or. (nproc == 16))
     &        .and. usescet .and. abovecut ) then
! for W+jet process in above-cut SCET calculation, leave as-is
          continue
        else
          write(6,*) '* Forced to be diagonal for simplicity in W + jets *'
          write(6,*) '****************************************************'
        endif
      endif
 47   format(' *      Vud=',g10.5,'Vus=',g10.5,'Vub=',g10.5,'  *')
 48   format(' *      Vcd=',g10.5,'Vcs=',g10.5,'Vcb=',g10.5,'  *')
      endif      

c--- special write-out for stop+b case
      if ((verbose .and. first .and. rank == 0) .and. (initrenscale_L > 0._dp)) then
      write(6,*)
      write(6,*) '************* Strong coupling, alpha_s  ************'
      write(6,*) '*                                                  *'
      write(6,49) 'alpha_s (zmass)    ',amz
      write(6,49) 'alpha_s (hvy scale)',as_H
      write(6,49) 'alpha_s (lgt scale)',as_L
      write(6,50) ' (using ',nlooprun,'-loop running of alpha_s)'  
      write(6,*) '****************************************************'
      else if (verbose .and. first .and. rank ==0 .and. scale > 0._dp) then
      write(6,*)
      write(6,*) '************* Strong coupling, alpha_s  ************'
      write(6,*) '*                                                  *'
      if (dynamicscale .eqv. .false.) then
      write(6,49) 'alpha_s (scale)',gsq/fourpi
      write(6,49) 'alpha_s (zmass)',amz
      else
      write(6,*) '*  Dynamic scale - alpha_s changed event-by-event  *'
      write(6,49) 'alpha_s (zmass)',amz
      endif
      write(6,50) ' (using ',nlooprun,'-loop running of alpha_s)'  
      write(6,*) '****************************************************'
 49   format(' *  ',a20,f12.8,16x,'*')
 50   format(' *  ',6x,a8,i1,a25,8x,'*')
      endif

      if (verbose .and. first .and. rank == 0 .and. doscalevar) then
      write(6,*)
      write(6,*) '****************************************************'
      write(6,*) '*                                                  *'
      write(6,*) '*     Computing scale variation in histograms      *'
      write(6,51) maxscalevar
      write(6,*) '*                                                  *'
      write(6,*) '****************************************************'
 51   format(' *    using ',i1,'-point variation by a factor of two    *')
      endif

      first = .false.
      
      return
      end
