!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine gen_Vphotons_jets(r,nphots,njets,p,wt,*)
      implicit none
      include 'src/Inc/types.f'
!---- generate phase space for 2-->2+nphots+njets process
!----   with (34) being a vector boson
!----   and 5,..,4+nphots the photons
!----   and 5+nphots,..,4+nphots+njets the jets
!----
!---- This routine is adapted from gen_njets.f
!----
!---- r(mxdim),p1(4),p2(4) are inputs reversed in sign 
!---- from physical values 
!---- phase space for -p1-p2 --> p3+p4+p5+p6
!---- with all 2 pi's (ie 1/(2*pi)^(4+2n))
!----
!---- if 'nodecay' is true, then the vector boson decay into massless
!---- particles is not included and 2 less integration variables
!---- are required
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/mxdim.f'
      include 'src/Inc/limits.f'
      include 'src/Inc/nodecay.f'
      include 'src/Inc/leptcuts.f'
      include 'src/Inc/reset.f'
      include 'src/Inc/breit.f'
      include 'src/Inc/kpart.f'
      include 'src/Inc/kprocess.f'
      include 'src/Inc/ipsgen.f'
      include 'src/Inc/x1x2.f'
      include 'src/Inc/energy.f'
      include 'src/Inc/first.f'
      real(dp):: r(mxdim),rdk1,rdk2,mflatsq
      real(dp):: p(mxpart,4),p3(4),p34(4),psumjet(4),pcm(4),Q(4)
      real(dp):: wt,dot,s345,s346
      real(dp):: hmin,hmax,delh,h,pt,etamax,etamin
      real(dp):: y,sinhy,coshy,phi,mv2,wtbw,mjets
      real(dp):: ybar,ptsumjet2,ycm,sumpst,q0st,rshat
      real(dp):: costh,sinth,dely,xjac
      real(dp):: ptjetmin,etajetmin,etajetmax,pbreak
      real(dp):: plstar,estar,plstarsq,y5starmax,y5starmin
      integer:: j,nu,nphots,njets,nphotsjets,ijet
      
!--- total number of photons and jets
      nphotsjets=nphots+njets

      p(:,:)=0._dp
      Q(:)=0._dp
      psumjet(:)=0._dp
      pcm(:)=0._dp

      wt=2._dp*pi
                       
      do ijet=1,nphotsjets
!--- generate the pt of jet number ijet
!--- rapidity limited by E=pT*coshy
        wt=wt/16._dp/pi**3

            pt = r(ijet)**2*sqrts/2._dp
            xjac = pt*sqrts/2._dp*2._dp*r(ijet)

        wt=wt*xjac
 
        etamax=sqrts/2._dp/pt
        if (etamax**2 <= 1._dp) then
          write(6,*) 'etamax**2 <= 1._dp in gen_phots_jets.f',etamax**2 
          wt=0._dp
          return 1
        endif
        etamax=log(etamax+sqrt(etamax**2-1._dp))
        
        etamax=min(etamax,20d0)
        y=etamax*(2._dp*r(nphotsjets+ijet)-1._dp)
        wt=wt*2._dp*etamax
        
        sinhy=sinh(y)
        coshy=sqrt(1._dp+sinhy**2)
        
        p(4+ijet,4)=pt*coshy
        
        phi=2._dp*pi*r(2*nphotsjets+ijet)
        wt=wt*2._dp*pi
        
        p(4+ijet,1)=pt*cos(phi)
        p(4+ijet,2)=pt*sin(phi)
        p(4+ijet,3)=pt*sinhy
        
        do nu=1,4
          psumjet(nu)=psumjet(nu)+p(4+ijet,nu)
        enddo
      enddo
      

      call breitw(r(3*nphotsjets+1),wsqmin,wsqmax,mass3,width3,mv2,wtbw)
      wt=wt*wtbw/2._dp/pi
      mjets=psumjet(4)**2-psumjet(1)**2-psumjet(2)**2-psumjet(3)**2
      mjets=sqrt(abs(mjets))
      
      ybar=0.5_dp*log((psumjet(4)+psumjet(3))/(psumjet(4)-psumjet(3)))
      ptsumjet2=psumjet(1)**2+psumjet(2)**2
      plstarsq=((sqrts**2-mv2-mjets**2)**2
     & -4._dp*(mjets**2*mv2+ptsumjet2*sqrts**2))/(4._dp*sqrts**2)
      if (plstarsq <= 0._dp) then
        wt=0._dp
        return 1
      endif
      plstar=sqrt(plstarsq)
      Estar=sqrt(plstarsq+ptsumjet2+mjets**2)
      y5starmax=0.5_dp*log((Estar+plstar)/(Estar-plstar))
      y5starmin=-y5starmax

      etamax=ybar-y5starmin
      etamin=ybar-y5starmax
      dely=etamax-etamin
      ycm=etamin+r(3*nphotsjets+2)*dely     
      sinhy=sinh(ycm)
      coshy=sqrt(1._dp+sinhy**2)
      
!--- now make the initial state momenta
      sumpst=ptsumjet2+(psumjet(3)*coshy-psumjet(4)*sinhy)**2
      q0st=sqrt(mv2+sumpst)
      rshat=q0st+sqrt(mjets**2+sumpst)
      pcm(4)=rshat*coshy
      pcm(3)=rshat*sinhy
            
      xx(1)=(pcm(4)+pcm(3))/sqrts
      xx(2)=(pcm(4)-pcm(3))/sqrts
      
      if   ((xx(1) > 1._dp) .or. (xx(2) > 1._dp)) then
         wt=0._dp
         return 1
      endif 
      
      wt=wt*dely
      do j=1,4
        Q(j)=pcm(j)-psumjet(j)
      enddo
      
      p(1,4)=-xx(1)*sqrts/2._dp
      p(1,3)=p(1,4)
      p(2,4)=-xx(2)*sqrts/2._dp
      p(2,3)=-p(2,4)
      
      wt=wt*rshat/(sqrts**2*q0st)
      
!--- dummy values if there's no decay
      if (nodecay) then
        rdk1=0.5_dp
        rdk2=0.5_dp
      else
        rdk1=r(3*nphotsjets+3)
        rdk2=r(3*nphotsjets+4)
      endif
      
!--- decay boson into leptons, in boson rest frame
      costh=2._dp*rdk1-1._dp
      sinth=sqrt(1._dp-costh**2)
      phi=2._dp*pi*rdk2
      p34(4)=sqrt(mv2)/2._dp
      p34(1)=p34(4)*sinth*cos(phi)
      p34(2)=p34(4)*sinth*sin(phi)
      p34(3)=p34(4)*costh
      
!--- boost into lab frame    
      call boost(sqrt(mv2),Q,p34,p3)
      do j=1,4
      p(3,j)=p3(j)
      p(4,j)=Q(j)-p(3,j)
      enddo

      wt=wt/8._dp/pi
      
      return
      end
      
      
      
      
      
      
      
      
      
      
      
