!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine ggttz(denu,denb,denq1,denq2,dena1,dena2,propz,wtgg)
      implicit none
      include 'src/Inc/types.f'
      
************************************************************************
*     Author: R.K. Ellis                                               *
*     May, 2011                                                        *
*     calculate the Born matrix element squared                        *
*     for the process                                                  *
*     My notation                                                      *
*     g(-p1) + g(-p2)=nu(p3)+e+(p4)+b(p5)+bbar(p6)+e-(p7)+nubar(p8)    *
*     +e^-(p9)+e^+(p10)                                                *
*                                                                      *
*  z=9+10                                                              *
*  q=3+4+5                                                             *
*  u=3+4+5+z                                                           *
*  a=-6-7-8                                                            *
*  b=-6-7-8-z                                                          *
*  since momenta 3,5,6,8 are not needed they are reused to             *
*  represent the de-massified vectors.                                 *
*  Thus q4 = q de-massified wrt p4 etc;                                *
*  this vector has been stored in wrapper routine (qqb_ttz) in p(3,mu) *
*  q4(mu)=q(mu)-qsq/2/p4Dq*p4(mu)-->p(3,mu)                            *
*  a7(mu)=a(mu)-asq/2/p7Da*p7(mu)-->p(5,mu)                            *
*  u1(mu)=u(mu)-usq/2/p1Du*p1(mu)-->p(6,mu)                            *
*  u2(mu)=u(mu)-usq/2/p2Du*p2(mu)-->p(8,mu)                            *
*  b1(mu)=b(mu)-bsq/2/p1Db*p1(mu)-->p(11,mu)                           *
*  b2(mu)=b(mu)-bsq/2/p2Db*p2(mu)-->p(12,mu)                           *
*  Formula generated by the program ggttz.frm                          *
*  First two indices of mab,mba are gluon polarizations                *
*  last two indices of coupling to quarks L and R and leptons le,re    *
************************************************************************
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zcouple.f'
      include 'src/Inc/ewcharge.f'
      include 'src/Inc/zprods_com.f'
      include 'src/Inc/masses.f'
      include 'src/Inc/topzlabels.f'
      complex(dp)::mab(2,2,2,2),mba(2,2,2,2),xab(2,2,2),xba(2,2,2),
     & propz
      real(dp):: denu,denb,denq1,denq2,dena1,dena2,wtgg,s12
      integer:: h1,h2,h3
      s12=real(za(p1,p2)*zb(p2,p1))
      mab(1,1,1,1)= + s12**(-2)*denb**(-1)*denq1**(-1) * ( za(p1,p2)**2
     &    *za(p1,q4)*za(p2,b2)*za(p7,a7)*za(p9,b1)*zb(p1,b1)*zb(p2,b2)*
     &    zb(p4,q4)*zb(p10,a7) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*denb**(-1) * (  - za(p1,
     &    p2)**2*za(p1,q4)*za(p7,a7)*za(p9,b1)*zb(p1,b1)*zb(p4,q4)*zb(
     &    p10,a7) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,p2)**2*za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,
     &    a7)*zb(p2,u2)*zb(p4,q4)*zb(p10,u1) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*denu**(-1) * (  - za(p1,
     &    p2)**2*za(p1,u1)*za(p7,a7)*za(p9,q4)*zb(p1,a7)*zb(p4,q4)*zb(
     &    p10,u1) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,p2)**3*za(p1,q4)*za(p7,a7)*za(p9,b2)*zb(p1,p10)*zb(p1
     &    ,a7)*zb(p2,b2)*zb(p4,q4) + za(p1,p2)**2*za(p1,q4)*za(p2,u2)*
     &    za(p7,a7)*za(p9,b2)*zb(p1,a7)*zb(p2,b2)*zb(p4,q4)*zb(p10,u2)
     &     )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p9,b1)*zb(p1,b1)*
     &    zb(p2,p4)*zb(p10,a7) - za(p1,p2)**2*za(p1,b1)*za(p2,p9)*za(p7
     &    ,a7)*zb(p1,b1)*zb(p2,p4)*zb(p10,a7) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p2)**2*za(p1,p9)*za(p7,a7)*zb(p1,p4)*zb(p10,a7) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p9,q4)*zb(p1,a7)*
     &    zb(p2,p10)*zb(p4,q4) - za(p1,p2)**2*za(p1,u1)*za(p2,p7)*za(p9
     &    ,q4)*zb(p1,u1)*zb(p2,p10)*zb(p4,q4) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p2)**2*za(p1,p7)*za(p9,q4)*zb(p1,p10)*zb(p4,q4) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p1,p9)*za(p7,a7)*zb(p1,p10)*
     &    zb(p1,a7)*zb(p2,p4) - za(p1,p2)**2*za(p1,p9)*za(p2,u2)*za(p7,
     &    a7)*zb(p1,a7)*zb(p2,p4)*zb(p10,u2) - za(p1,p2)**2*za(p1,q4)*
     &    za(p2,p7)*za(p9,b2)*zb(p1,p10)*zb(p2,b2)*zb(p4,q4) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-2)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p2)**2*za(p1,p9)*za(p2,p7)*zb(p1,p10)*zb(
     &    p2,p4) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * (  - za(p1,p2)**2*za(p1,u1)*za(p7,a7)*za(p9,q4)*zb(p1,a7)*zb(
     &    p4,q4)*zb(p10,u1) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p2)**2*za(p2,p9)*za(p7,a7)*zb(p2,p4)*
     &    zb(p10,a7) + za(p1,p2)*za(p1,q4)*za(p2,p9)*za(p7,a7)*zb(p4,q4
     &    )*zb(p10,a7) )
      mab(1,1,1,1) = mab(1,1,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)*za(p1,u1)*za(p2,p7)*za(p9,q4)*zb(p4,q4
     &    )*zb(p10,u1) )

      mab(1,1,1,2)= + s12**(-2)*denb**(-1)*denq1**(-1) * ( za(p1,p2)**2
     &    *za(p1,q4)*za(p2,b2)*za(p7,a7)*za(p10,b1)*zb(p1,b1)*zb(p2,b2)
     &    *zb(p4,q4)*zb(p9,a7) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*denb**(-1) * (  - za(p1,
     &    p2)**2*za(p1,q4)*za(p7,a7)*za(p10,b1)*zb(p1,b1)*zb(p4,q4)*zb(
     &    p9,a7) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,p2)**2*za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1
     &    ,a7)*zb(p2,u2)*zb(p4,q4)*zb(p9,u1) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*denu**(-1) * (  - za(p1,
     &    p2)**2*za(p1,u1)*za(p7,a7)*za(p10,q4)*zb(p1,a7)*zb(p4,q4)*zb(
     &    p9,u1) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,p2)**3*za(p1,q4)*za(p7,a7)*za(p10,b2)*zb(p1,p9)*zb(p1
     &    ,a7)*zb(p2,b2)*zb(p4,q4) + za(p1,p2)**2*za(p1,q4)*za(p2,u2)*
     &    za(p7,a7)*za(p10,b2)*zb(p1,a7)*zb(p2,b2)*zb(p4,q4)*zb(p9,u2)
     &     )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p10,b1)*zb(p1,b1)*
     &    zb(p2,p4)*zb(p9,a7) - za(p1,p2)**2*za(p1,b1)*za(p2,p10)*za(p7
     &    ,a7)*zb(p1,b1)*zb(p2,p4)*zb(p9,a7) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p2)**2*za(p1,p10)*za(p7,a7)*zb(p1,p4)*zb(p9,a7) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p10,q4)*zb(p1,a7)*
     &    zb(p2,p9)*zb(p4,q4) - za(p1,p2)**2*za(p1,u1)*za(p2,p7)*za(p10
     &    ,q4)*zb(p1,u1)*zb(p2,p9)*zb(p4,q4) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p2)**2*za(p1,p7)*za(p10,q4)*zb(p1,p9)*zb(p4,q4) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p1,p10)*za(p7,a7)*zb(p1,p9)*
     &    zb(p1,a7)*zb(p2,p4) - za(p1,p2)**2*za(p1,p10)*za(p2,u2)*za(p7
     &    ,a7)*zb(p1,a7)*zb(p2,p4)*zb(p9,u2) - za(p1,p2)**2*za(p1,q4)*
     &    za(p2,p7)*za(p10,b2)*zb(p1,p9)*zb(p2,b2)*zb(p4,q4) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-2)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p2)**2*za(p1,p10)*za(p2,p7)*zb(p1,p9)*zb(
     &    p2,p4) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * (  - za(p1,p2)**2*za(p1,u1)*za(p7,a7)*za(p10,q4)*zb(p1,a7)*
     &    zb(p4,q4)*zb(p9,u1) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p2)**2*za(p2,p10)*za(p7,a7)*zb(p2,p4)*
     &    zb(p9,a7) + za(p1,p2)*za(p1,q4)*za(p2,p10)*za(p7,a7)*zb(p4,q4
     &    )*zb(p9,a7) )
      mab(1,1,1,2) = mab(1,1,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)*za(p1,u1)*za(p2,p7)*za(p10,q4)*zb(p4,
     &    q4)*zb(p9,u1) )

      mab(1,1,2,1)= + s12**(-2)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,p2)**2*za(p1,q4)*za(p2,b2)*za(p7,p9)*zb(p1,p10)*zb(p2,b2)*
     &    zb(p4,q4) + za(p1,p2)**2*za(p1,b1)*za(p2,b2)*za(p7,p9)*zb(p1,
     &    b1)*zb(p2,p4)*zb(p10,b2) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p2)**2*za(p1,q4)*za(p7,p9)*zb(p1,p10)*zb(p4,q4) - za(p1,p2
     &    )**2*za(p1,b1)*za(p7,p9)*zb(p1,p4)*zb(p10,b1) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p9,u2)*zb(p1,a7)*zb(p2
     &    ,u2)*zb(p4,p10) - za(p1,p2)**2*za(p1,p9)*za(p2,u2)*za(p7,a7)*
     &    zb(p1,a7)*zb(p2,u2)*zb(p4,p10) + za(p1,p2)**2*za(p1,u1)*za(p2
     &    ,p7)*za(p9,u2)*zb(p1,u1)*zb(p2,u2)*zb(p4,p10) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p7)*za(p9,u1)*zb(p1,u1)*zb(p4,p10) + za(p1
     &    ,p2)**2*za(p1,p9)*za(p7,a7)*zb(p1,a7)*zb(p4,p10) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p2,p7)*za(p9,u1)*zb(p1,u1)*
     &    zb(p2,p4)*zb(p2,p10) + za(p1,p2)**3*za(p2,p9)*za(p7,a7)*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p2,p10) + za(p1,p2)**2*za(p1,q4)*za(p2,p7)*
     &    za(p9,u1)*zb(p1,u1)*zb(p2,p10)*zb(p4,q4) - za(p1,p2)**2*za(p1
     &    ,q4)*za(p2,p9)*za(p7,a7)*zb(p1,a7)*zb(p2,p10)*zb(p4,q4) - za(
     &    p1,p2)**2*za(p1,b1)*za(p2,p7)*za(p9,u1)*zb(p1,u1)*zb(p2,p4)*
     &    zb(p10,b1) + za(p1,p2)**2*za(p1,b1)*za(p2,p9)*za(p7,a7)*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p10,b1) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-2)*mt**4*denb**(-1)*
     & denq1**(-1) * ( za(p1,p2)**3*za(p7,p9)*zb(p1,p10)*zb(p2,p4) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * ( za(p1,p2)**2*za(p2,b2)*za(p7,p9)*zb(p2,p4)*zb(
     &    p10,b2) - za(p1,p2)*za(p1,q4)*za(p2,b2)*za(p7,p9)*zb(p4,q4)*
     &    zb(p10,b2) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)**2*za(p1,p9)*za(p7,a7)*zb(p1,a7)*zb(p4
     &    ,p10) )
      mab(1,1,2,1) = mab(1,1,2,1) + s12**(-1)*mt**4*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p1,p9)*za(p2,p7)*zb(p4,p10) )

      mab(1,1,2,2)= + s12**(-2)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,p2)**2*za(p1,q4)*za(p2,b2)*za(p7,p10)*zb(p1,p9)*zb(p2,b2)*
     &    zb(p4,q4) + za(p1,p2)**2*za(p1,b1)*za(p2,b2)*za(p7,p10)*zb(p1
     &    ,b1)*zb(p2,p4)*zb(p9,b2) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p2)**2*za(p1,q4)*za(p7,p10)*zb(p1,p9)*zb(p4,q4) - za(p1,p2
     &    )**2*za(p1,b1)*za(p7,p10)*zb(p1,p4)*zb(p9,b1) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p10,u2)*zb(p1,a7)*zb(
     &    p2,u2)*zb(p4,p9) - za(p1,p2)**2*za(p1,p10)*za(p2,u2)*za(p7,a7
     &    )*zb(p1,a7)*zb(p2,u2)*zb(p4,p9) + za(p1,p2)**2*za(p1,u1)*za(
     &    p2,p7)*za(p10,u2)*zb(p1,u1)*zb(p2,u2)*zb(p4,p9) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p7)*za(p10,u1)*zb(p1,u1)*zb(p4,p9) + za(p1
     &    ,p2)**2*za(p1,p10)*za(p7,a7)*zb(p1,a7)*zb(p4,p9) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)**3*za(p2,p7)*za(p10,u1)*zb(p1,u1)*
     &    zb(p2,p4)*zb(p2,p9) + za(p1,p2)**3*za(p2,p10)*za(p7,a7)*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p2,p9) + za(p1,p2)**2*za(p1,q4)*za(p2,p7)*
     &    za(p10,u1)*zb(p1,u1)*zb(p2,p9)*zb(p4,q4) - za(p1,p2)**2*za(p1
     &    ,q4)*za(p2,p10)*za(p7,a7)*zb(p1,a7)*zb(p2,p9)*zb(p4,q4) - za(
     &    p1,p2)**2*za(p1,b1)*za(p2,p7)*za(p10,u1)*zb(p1,u1)*zb(p2,p4)*
     &    zb(p9,b1) + za(p1,p2)**2*za(p1,b1)*za(p2,p10)*za(p7,a7)*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p9,b1) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-2)*mt**4*denb**(-1)*
     & denq1**(-1) * ( za(p1,p2)**3*za(p7,p10)*zb(p1,p9)*zb(p2,p4) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * ( za(p1,p2)**2*za(p2,b2)*za(p7,p10)*zb(p2,p4)*zb(
     &    p9,b2) - za(p1,p2)*za(p1,q4)*za(p2,b2)*za(p7,p10)*zb(p4,q4)*
     &    zb(p9,b2) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p2)**2*za(p1,p10)*za(p7,a7)*zb(p1,a7)*zb(
     &    p4,p9) )
      mab(1,1,2,2) = mab(1,1,2,2) + s12**(-1)*mt**4*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p1,p10)*za(p2,p7)*zb(p4,p9) )

      mab(2,2,1,1)= + s12**(-2)*denb**(-1)*denq1**(-1) * ( za(p1,b1)*
     &    za(p2,q4)*za(p7,a7)*za(p9,b2)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,b2
     &    )*zb(p4,q4)*zb(p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*denb**(-1) * (  - za(p1,
     &    q4)*za(p7,a7)*za(p9,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4,q4)*zb(
     &    p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,
     &    u1)*zb(p2,a7)*zb(p4,q4)*zb(p10,u2) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*denu**(-1) * (  - za(p1,
     &    u1)*za(p7,a7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,q4)*zb(
     &    p10,u1) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p2,p9)*za(p2,q4)*za(p7,a7)*zb(p1,p2)**3*zb(p2,
     &    a7)*zb(p4,q4)*zb(p10,u1) + za(p1,u1)*za(p2,q4)*za(p7,a7)*za(
     &    p9,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,a7)*zb(p4,q4)*zb(p10,u1)
     &     )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**2*denb**(-1)*
     & denq1**(-1) * ( za(p1,p9)*za(p2,q4)*za(p7,a7)*zb(p1,p2)**3*zb(p4
     &    ,q4)*zb(p10,a7) - za(p1,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p2)**2*
     &    zb(p1,p4)*zb(p2,b2)*zb(p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p9)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,u2)*za(p9,q4)*zb(p1,p2)**3*zb(p4
     &    ,q4)*zb(p10,u2) - za(p1,p7)*za(p2,u2)*za(p9,q4)*zb(p1,p2)**2*
     &    zb(p1,p10)*zb(p2,u2)*zb(p4,q4) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,p10)*zb(p4,q4) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p2,p9)*za(p2,q4)*zb(p1,p2)**3*
     &    zb(p2,p10)*zb(p4,q4) - za(p1,p7)*za(p2,q4)*za(p9,b1)*zb(p1,p2
     &    )**2*zb(p1,b1)*zb(p2,p10)*zb(p4,q4) - za(p1,u1)*za(p2,p9)*za(
     &    p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p2,a7)*zb(p10,u1) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-2)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,p9)*zb(p1,p2)**2*zb(p1,p4)*zb(p2
     &    ,p10) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-1)*denb**(-1)*denq1**(-1)
     &  * ( za(p2,q4)*za(p7,a7)*za(p9,b2)*zb(p1,p2)**2*zb(p2,b2)*zb(p4,
     &    q4)*zb(p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p7,a7)*za(p9,b2)*zb(p1,p2)*zb(p1,p4)*zb(p2
     &    ,b2)*zb(p10,a7) )
      mab(2,2,1,1) = mab(2,2,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,p10)*zb(
     &    p4,q4) - za(p7,a7)*za(p9,q4)*zb(p1,p2)*zb(p1,p10)*zb(p2,a7)*
     &    zb(p4,q4) )

      mab(2,2,1,2)= + s12**(-2)*denb**(-1)*denq1**(-1) * ( za(p1,b1)*
     &    za(p2,q4)*za(p7,a7)*za(p10,b2)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,
     &    b2)*zb(p4,q4)*zb(p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*denb**(-1) * (  - za(p1,
     &    q4)*za(p7,a7)*za(p10,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4,q4)*zb(
     &    p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1,p2)**2*zb(p1
     &    ,u1)*zb(p2,a7)*zb(p4,q4)*zb(p9,u2) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*denu**(-1) * (  - za(p1,
     &    u1)*za(p7,a7)*za(p10,q4)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,q4)*zb(
     &    p9,u1) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p2,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p2)**3*zb(p2
     &    ,a7)*zb(p4,q4)*zb(p9,u1) + za(p1,u1)*za(p2,q4)*za(p7,a7)*za(
     &    p10,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,a7)*zb(p4,q4)*zb(p9,u1)
     &     )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**2*denb**(-1)*
     & denq1**(-1) * ( za(p1,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p2)**3*zb(
     &    p4,q4)*zb(p9,a7) - za(p1,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p2)**
     &    2*zb(p1,p4)*zb(p2,b2)*zb(p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,p10)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,u2)*za(p10,q4)*zb(p1,p2)**3*zb(
     &    p4,q4)*zb(p9,u2) - za(p1,p7)*za(p2,u2)*za(p10,q4)*zb(p1,p2)**
     &    2*zb(p1,p9)*zb(p2,u2)*zb(p4,q4) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p7)*za(p10,q4)*zb(p1,p2)**2*zb(p1,p9)*zb(p4,q4) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p2,p10)*za(p2,q4)*zb(p1,p2)**3*
     &    zb(p2,p9)*zb(p4,q4) - za(p1,p7)*za(p2,q4)*za(p10,b1)*zb(p1,p2
     &    )**2*zb(p1,b1)*zb(p2,p9)*zb(p4,q4) - za(p1,u1)*za(p2,p10)*za(
     &    p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p2,a7)*zb(p9,u1) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-2)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,p10)*zb(p1,p2)**2*zb(p1,p4)*zb(
     &    p2,p9) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-1)*denb**(-1)*denq1**(-1)
     &  * ( za(p2,q4)*za(p7,a7)*za(p10,b2)*zb(p1,p2)**2*zb(p2,b2)*zb(p4
     &    ,q4)*zb(p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p7,a7)*za(p10,b2)*zb(p1,p2)*zb(p1,p4)*zb(
     &    p2,b2)*zb(p9,a7) )
      mab(2,2,1,2) = mab(2,2,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p10,q4)*zb(p1,p2)**2*zb(p1,p9)*zb(
     &    p4,q4) - za(p7,a7)*za(p10,q4)*zb(p1,p2)*zb(p1,p9)*zb(p2,a7)*
     &    zb(p4,q4) )

      mab(2,2,2,1)= + s12**(-2)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,b1)*za(p2,q4)*za(p7,p9)*zb(p1,p2)**3*zb(p4,q4)*zb(p10,b1)
     &     - za(p1,b1)*za(p2,q4)*za(p7,p9)*zb(p1,p2)**2*zb(p1,b1)*zb(p2
     &    ,p10)*zb(p4,q4) + za(p1,b1)*za(p2,b2)*za(p7,p9)*zb(p1,p2)**2*
     &    zb(p1,p4)*zb(p2,b2)*zb(p10,b1) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,q4)*za(p7,p9)*zb(p1,p2)**2*zb(p1,p10)*zb(p4,q4) - za(p1,b1
     &    )*za(p7,p9)*zb(p1,p2)**2*zb(p1,p4)*zb(p10,b1) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,u2)*za(p9,u1)*zb(p1,p2)**2*zb(p1
     &    ,u1)*zb(p2,u2)*zb(p4,p10) - za(p1,u1)*za(p2,p9)*za(p7,a7)*zb(
     &    p1,p2)**2*zb(p1,u1)*zb(p2,a7)*zb(p4,p10) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p7)*za(p9,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p4,p10) + za(p1
     &    ,p9)*za(p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,p10) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,p9)*za(p2,q4)*zb(p1,p2)**3*zb(p1
     &    ,p10)*zb(p4,q4) - za(p1,p7)*za(p1,p9)*za(p2,b2)*zb(p1,p2)**3*
     &    zb(p1,p4)*zb(p10,b2) + za(p1,p7)*za(p2,q4)*za(p9,u2)*zb(p1,p2
     &    )**2*zb(p1,p10)*zb(p2,u2)*zb(p4,q4) - za(p1,p7)*za(p2,b2)*za(
     &    p9,u2)*zb(p1,p2)**2*zb(p1,p4)*zb(p2,u2)*zb(p10,b2) - za(p1,p9
     &    )*za(p2,q4)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p10)*zb(p2,a7)*zb(p4
     &    ,q4) + za(p1,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p4)*
     &    zb(p2,a7)*zb(p10,b2) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-2)*mt**4*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p2,p9)*zb(p1,p2)**3*zb(p4,p10) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p2,q4)*za(p7,p9)*zb(p1,p2)**2*zb(p2,p10)*
     &    zb(p4,q4) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p9,u1)*zb(p1,p2)**2*zb(p1,u1)*
     &    zb(p4,p10) + za(p7,a7)*za(p9,u1)*zb(p1,p2)*zb(p1,u1)*zb(p2,a7
     &    )*zb(p4,p10) )
      mab(2,2,2,1) = mab(2,2,2,1) + s12**(-1)*mt**4*denb**(-1)*
     & denq1**(-1) * ( za(p7,p9)*zb(p1,p2)*zb(p1,p4)*zb(p2,p10) )

      mab(2,2,2,2)= + s12**(-2)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,b1)*za(p2,q4)*za(p7,p10)*zb(p1,p2)**3*zb(p4,q4)*zb(p9,b1)
     &     - za(p1,b1)*za(p2,q4)*za(p7,p10)*zb(p1,p2)**2*zb(p1,b1)*zb(
     &    p2,p9)*zb(p4,q4) + za(p1,b1)*za(p2,b2)*za(p7,p10)*zb(p1,p2)**
     &    2*zb(p1,p4)*zb(p2,b2)*zb(p9,b1) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-2)*mt**2*denb**(-1) * ( za(
     &    p1,q4)*za(p7,p10)*zb(p1,p2)**2*zb(p1,p9)*zb(p4,q4) - za(p1,b1
     &    )*za(p7,p10)*zb(p1,p2)**2*zb(p1,p4)*zb(p9,b1) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p2,u2)*za(p10,u1)*zb(p1,p2)**2*zb(
     &    p1,u1)*zb(p2,u2)*zb(p4,p9) - za(p1,u1)*za(p2,p10)*za(p7,a7)*
     &    zb(p1,p2)**2*zb(p1,u1)*zb(p2,a7)*zb(p4,p9) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p7)*za(p10,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p4,p9) + za(p1
     &    ,p10)*za(p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,p9) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-2)*mt**2*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,p10)*za(p2,q4)*zb(p1,p2)**3*zb(
     &    p1,p9)*zb(p4,q4) - za(p1,p7)*za(p1,p10)*za(p2,b2)*zb(p1,p2)**
     &    3*zb(p1,p4)*zb(p9,b2) + za(p1,p7)*za(p2,q4)*za(p10,u2)*zb(p1,
     &    p2)**2*zb(p1,p9)*zb(p2,u2)*zb(p4,q4) - za(p1,p7)*za(p2,b2)*
     &    za(p10,u2)*zb(p1,p2)**2*zb(p1,p4)*zb(p2,u2)*zb(p9,b2) - za(p1
     &    ,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p9)*zb(p2,a7)*
     &    zb(p4,q4) + za(p1,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p2)**2*zb(p1
     &    ,p4)*zb(p2,a7)*zb(p9,b2) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-2)*mt**4*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p2,p10)*zb(p1,p2)**3*zb(p4,p9) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p2,q4)*za(p7,p10)*zb(p1,p2)**2*zb(p2,p9)*
     &    zb(p4,q4) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p10,u1)*zb(p1,p2)**2*zb(p1,u1)*
     &    zb(p4,p9) + za(p7,a7)*za(p10,u1)*zb(p1,p2)*zb(p1,u1)*zb(p2,a7
     &    )*zb(p4,p9) )
      mab(2,2,2,2) = mab(2,2,2,2) + s12**(-1)*mt**4*denb**(-1)*
     & denq1**(-1) * ( za(p7,p10)*zb(p1,p2)*zb(p1,p4)*zb(p2,p9) )

      mab(1,2,1,1)= + s12**(-1)*denb**(-1)*denq1**(-1) * ( za(p1,q4)*
     &    za(p1,b2)*za(p7,a7)*za(p9,b2)*zb(p2,b2)**2*zb(p4,q4)*zb(p10,
     &    a7) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p1,u2)*za(p7,a7)*za(p9,q4)*zb(p2,a7)*zb(p2,u2)
     &    *zb(p4,q4)*zb(p10,u1) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,q4)*za(p1,u1)*za(p7,a7)*za(p9,b2)*zb(p2,a7)*zb(p2,b2)
     &    *zb(p4,q4)*zb(p10,u1) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p9)*za(p1,b1)*za(p7,a7)*zb(p2,p4)*zb(p2
     &    ,b1)*zb(p10,a7) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p1,u1)*za(p9,q4)*zb(p2,p10)*zb(
     &    p2,u1)*zb(p4,q4) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p1,q4)*za(p9,b2)*zb(p2,p10)*zb(
     &    p2,b2)*zb(p4,q4) - za(p1,p9)*za(p1,u1)*za(p7,a7)*zb(p2,p4)*
     &    zb(p2,a7)*zb(p10,u1) )
      mab(1,2,1,1) = mab(1,2,1,1) + s12**(-1)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,p9)*zb(p2,p4)*zb(p2,p10) )

      mab(1,2,1,2)= + s12**(-1)*denb**(-1)*denq1**(-1) * ( za(p1,q4)*
     &    za(p1,b2)*za(p7,a7)*za(p10,b2)*zb(p2,b2)**2*zb(p4,q4)*zb(p9,
     &    a7) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * ( za(p1,u1)*za(p1,u2)*za(p7,a7)*za(p10,q4)*zb(p2,a7)*zb(p2,u2
     &    )*zb(p4,q4)*zb(p9,u1) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,q4)*za(p1,u1)*za(p7,a7)*za(p10,b2)*zb(p2,a7)*zb(p2,b2
     &    )*zb(p4,q4)*zb(p9,u1) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p1,p10)*za(p1,b1)*za(p7,a7)*zb(p2,p4)*zb(
     &    p2,b1)*zb(p9,a7) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p1,u1)*za(p10,q4)*zb(p2,p9)*zb(
     &    p2,u1)*zb(p4,q4) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p7)*za(p1,q4)*za(p10,b2)*zb(p2,p9)*zb(
     &    p2,b2)*zb(p4,q4) - za(p1,p10)*za(p1,u1)*za(p7,a7)*zb(p2,p4)*
     &    zb(p2,a7)*zb(p9,u1) )
      mab(1,2,1,2) = mab(1,2,1,2) + s12**(-1)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,p10)*zb(p2,p4)*zb(p2,p9) )

      mab(1,2,2,1)= + s12**(-1)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,q4)*za(p1,b2)*za(p7,p9)*zb(p2,p10)*zb(p2,b2)*zb(p4,q4) + 
     &    za(p1,b1)**2*za(p7,p9)*zb(p2,p4)*zb(p2,b1)*zb(p10,b1) )
      mab(1,2,2,1) = mab(1,2,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,u1)*za(p9,u2)*zb(p2,u1)*zb(p2,u2
     &    )*zb(p4,p10) - za(p1,p9)*za(p1,u2)*za(p7,a7)*zb(p2,a7)*zb(p2,
     &    u2)*zb(p4,p10) )
      mab(1,2,2,1) = mab(1,2,2,1) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p1,p7)*za(p9,u2)*zb(p2,p4)*zb(p2
     &    ,p10)*zb(p2,u2) + za(p1,p2)*za(p1,p9)*za(p7,a7)*zb(p2,p4)*zb(
     &    p2,p10)*zb(p2,a7) + za(p1,p7)*za(p1,p9)*za(p1,q4)*zb(p1,p2)*
     &    zb(p2,p10)*zb(p4,q4) - za(p1,p7)*za(p1,p9)*za(p1,b1)*zb(p1,p2
     &    )*zb(p2,p4)*zb(p10,b1) + za(p1,p7)*za(p1,q4)*za(p9,u2)*zb(p2,
     &    p10)*zb(p2,u2)*zb(p4,q4) - za(p1,p7)*za(p1,b1)*za(p9,u2)*zb(
     &    p2,p4)*zb(p2,u2)*zb(p10,b1) - za(p1,p9)*za(p1,q4)*za(p7,a7)*
     &    zb(p2,p10)*zb(p2,a7)*zb(p4,q4) + za(p1,p9)*za(p1,b1)*za(p7,a7
     &    )*zb(p2,p4)*zb(p2,a7)*zb(p10,b1) )
      mab(1,2,2,1) = mab(1,2,2,1) + mt**2*denq1**(-1)*dena2**(-1) * ( 
     &    za(p1,p7)*za(p1,p9)*zb(p2,p4)*zb(p2,p10) )

      mab(1,2,2,2)= + s12**(-1)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p1,q4)*za(p1,b2)*za(p7,p10)*zb(p2,p9)*zb(p2,b2)*zb(p4,q4) + 
     &    za(p1,b1)**2*za(p7,p10)*zb(p2,p4)*zb(p2,b1)*zb(p9,b1) )
      mab(1,2,2,2) = mab(1,2,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p1,p7)*za(p1,u1)*za(p10,u2)*zb(p2,u1)*zb(p2,
     &    u2)*zb(p4,p9) - za(p1,p10)*za(p1,u2)*za(p7,a7)*zb(p2,a7)*zb(
     &    p2,u2)*zb(p4,p9) )
      mab(1,2,2,2) = mab(1,2,2,2) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p1,p7)*za(p10,u2)*zb(p2,p4)*zb(
     &    p2,p9)*zb(p2,u2) + za(p1,p2)*za(p1,p10)*za(p7,a7)*zb(p2,p4)*
     &    zb(p2,p9)*zb(p2,a7) + za(p1,p7)*za(p1,p10)*za(p1,q4)*zb(p1,p2
     &    )*zb(p2,p9)*zb(p4,q4) - za(p1,p7)*za(p1,p10)*za(p1,b1)*zb(p1,
     &    p2)*zb(p2,p4)*zb(p9,b1) + za(p1,p7)*za(p1,q4)*za(p10,u2)*zb(
     &    p2,p9)*zb(p2,u2)*zb(p4,q4) - za(p1,p7)*za(p1,b1)*za(p10,u2)*
     &    zb(p2,p4)*zb(p2,u2)*zb(p9,b1) - za(p1,p10)*za(p1,q4)*za(p7,a7
     &    )*zb(p2,p9)*zb(p2,a7)*zb(p4,q4) + za(p1,p10)*za(p1,b1)*za(p7,
     &    a7)*zb(p2,p4)*zb(p2,a7)*zb(p9,b1) )
      mab(1,2,2,2) = mab(1,2,2,2) + mt**2*denq1**(-1)*dena2**(-1) * ( 
     &    za(p1,p7)*za(p1,p10)*zb(p2,p4)*zb(p2,p9) )

      mab(2,1,1,1)= + s12**(-1)*denb**(-1)*denq1**(-1) * ( za(p2,q4)*
     &    za(p2,b1)*za(p7,a7)*za(p9,b1)*zb(p1,b1)**2*zb(p4,q4)*zb(p10,
     &    a7) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * ( za(p2,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,a7)*zb(p1,u1)
     &    *zb(p4,q4)*zb(p10,u2) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,p2)*za(p2,q4)*za(p7,a7)*za(p9,b1)*zb(p1,p10)*zb(p1,a7
     &    )*zb(p1,b1)*zb(p4,q4) + za(p2,p9)*za(p2,q4)*za(p2,u2)*za(p7,
     &    a7)*zb(p1,p2)*zb(p1,a7)*zb(p4,q4)*zb(p10,u2) + za(p2,q4)*za(
     &    p2,u2)*za(p7,a7)*za(p9,b1)*zb(p1,a7)*zb(p1,b1)*zb(p4,q4)*zb(
     &    p10,u2) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p2,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p4)*zb(p1
     &    ,b2)*zb(p10,a7) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p2,p7)*za(p2,u2)*za(p9,q4)*zb(p1,p10)*zb(
     &    p1,u2)*zb(p4,q4) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p2,p9)*za(p7,a7)*zb(p1,p4)*zb(p1
     &    ,p10)*zb(p1,a7) - za(p2,p7)*za(p2,p9)*za(p2,q4)*zb(p1,p2)*zb(
     &    p1,p10)*zb(p4,q4) - za(p2,p7)*za(p2,q4)*za(p9,b1)*zb(p1,p10)*
     &    zb(p1,b1)*zb(p4,q4) - za(p2,p9)*za(p2,u2)*za(p7,a7)*zb(p1,p4)
     &    *zb(p1,a7)*zb(p10,u2) )
      mab(2,1,1,1) = mab(2,1,1,1) + s12**(-1)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,p9)*zb(p1,p4)*zb(p1,p10) )
      mab(2,1,1,1) = mab(2,1,1,1) + denq1**(-1)*dena2**(-1) * (  - za(
     &    p2,p9)*za(p2,q4)*za(p7,a7)*zb(p1,p10)*zb(p1,a7)*zb(p4,q4) )

      mab(2,1,1,2)= + s12**(-1)*denb**(-1)*denq1**(-1) * ( za(p2,q4)*
     &    za(p2,b1)*za(p7,a7)*za(p10,b1)*zb(p1,b1)**2*zb(p4,q4)*zb(p9,
     &    a7) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*denu**(-1)*dena2**(-1)
     &  * ( za(p2,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1,a7)*zb(p1,u1
     &    )*zb(p4,q4)*zb(p9,u2) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*denq1**(-1)*dena2**(-1)
     &  * ( za(p1,p2)*za(p2,q4)*za(p7,a7)*za(p10,b1)*zb(p1,p9)*zb(p1,a7
     &    )*zb(p1,b1)*zb(p4,q4) + za(p2,p10)*za(p2,q4)*za(p2,u2)*za(p7,
     &    a7)*zb(p1,p2)*zb(p1,a7)*zb(p4,q4)*zb(p9,u2) + za(p2,q4)*za(p2
     &    ,u2)*za(p7,a7)*za(p10,b1)*zb(p1,a7)*zb(p1,b1)*zb(p4,q4)*zb(p9
     &    ,u2) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq1**(-1) * (  - za(p2,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p4)*zb(
     &    p1,b2)*zb(p9,a7) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * (  - za(p2,p7)*za(p2,u2)*za(p10,q4)*zb(p1,p9)*zb(
     &    p1,u2)*zb(p4,q4) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * (  - za(p1,p2)*za(p2,p10)*za(p7,a7)*zb(p1,p4)*zb(
     &    p1,p9)*zb(p1,a7) - za(p2,p7)*za(p2,p10)*za(p2,q4)*zb(p1,p2)*
     &    zb(p1,p9)*zb(p4,q4) - za(p2,p7)*za(p2,q4)*za(p10,b1)*zb(p1,p9
     &    )*zb(p1,b1)*zb(p4,q4) - za(p2,p10)*za(p2,u2)*za(p7,a7)*zb(p1,
     &    p4)*zb(p1,a7)*zb(p9,u2) )
      mab(2,1,1,2) = mab(2,1,1,2) + s12**(-1)*mt**4*denq1**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,p10)*zb(p1,p4)*zb(p1,p9) )
      mab(2,1,1,2) = mab(2,1,1,2) + denq1**(-1)*dena2**(-1) * (  - za(
     &    p2,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p9)*zb(p1,a7)*zb(p4,q4) )

      mab(2,1,2,1)= + s12**(-1)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p2,q4)*za(p2,b1)*za(p7,p9)*zb(p1,p10)*zb(p1,b1)*zb(p4,q4) + 
     &    za(p2,b2)**2*za(p7,p9)*zb(p1,p4)*zb(p1,b2)*zb(p10,b2) )
      mab(2,1,2,1) = mab(2,1,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,u2)*za(p9,u1)*zb(p1,u1)*zb(p1,u2
     &    )*zb(p4,p10) - za(p2,p9)*za(p2,u1)*za(p7,a7)*zb(p1,a7)*zb(p1,
     &    u1)*zb(p4,p10) )
      mab(2,1,2,1) = mab(2,1,2,1) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,q4)*za(p9,u1)*zb(p1,p10)*zb(p1,
     &    u1)*zb(p4,q4) - za(p2,p7)*za(p2,b2)*za(p9,u1)*zb(p1,p4)*zb(p1
     &    ,u1)*zb(p10,b2) - za(p2,p9)*za(p2,q4)*za(p7,a7)*zb(p1,p10)*
     &    zb(p1,a7)*zb(p4,q4) + za(p2,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p4)
     &    *zb(p1,a7)*zb(p10,b2) )

      mab(2,1,2,2)= + s12**(-1)*mt**2*denb**(-1)*denq1**(-1) * (  - za(
     &    p2,q4)*za(p2,b1)*za(p7,p10)*zb(p1,p9)*zb(p1,b1)*zb(p4,q4) + 
     &    za(p2,b2)**2*za(p7,p10)*zb(p1,p4)*zb(p1,b2)*zb(p9,b2) )
      mab(2,1,2,2) = mab(2,1,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,u2)*za(p10,u1)*zb(p1,u1)*zb(p1,
     &    u2)*zb(p4,p9) - za(p2,p10)*za(p2,u1)*za(p7,a7)*zb(p1,a7)*zb(
     &    p1,u1)*zb(p4,p9) )
      mab(2,1,2,2) = mab(2,1,2,2) + s12**(-1)*mt**2*denq1**(-1)*
     & dena2**(-1) * ( za(p2,p7)*za(p2,q4)*za(p10,u1)*zb(p1,p9)*zb(p1,
     &    u1)*zb(p4,q4) - za(p2,p7)*za(p2,b2)*za(p10,u1)*zb(p1,p4)*zb(
     &    p1,u1)*zb(p9,b2) - za(p2,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p9)*
     &    zb(p1,a7)*zb(p4,q4) + za(p2,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p4
     &    )*zb(p1,a7)*zb(p9,b2) )

      mba(1,1,1,1)= + s12**(-2)*denb**(-1)*denq2**(-1) * ( za(p1,p2)**2
     &    *za(p1,b1)*za(p2,q4)*za(p7,a7)*za(p9,b2)*zb(p1,b1)*zb(p2,b2)*
     &    zb(p4,q4)*zb(p10,a7) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*denb**(-1) * ( za(p1,p2)
     &    **2*za(p1,q4)*za(p7,a7)*za(p9,b1)*zb(p1,b1)*zb(p4,q4)*zb(p10,
     &    a7) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,p2)**2*za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,
     &    u1)*zb(p2,a7)*zb(p4,q4)*zb(p10,u2) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*denu**(-1) * ( za(p1,p2)
     &    **2*za(p1,u1)*za(p7,a7)*za(p9,q4)*zb(p1,a7)*zb(p4,q4)*zb(p10,
     &    u1) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)**3*za(p2,q4)*za(p7,a7)*za(p9,b1)*zb(p1,b1)*zb(
     &    p2,p10)*zb(p2,a7)*zb(p4,q4) + za(p1,p2)**2*za(p1,u1)*za(p2,q4
     &    )*za(p7,a7)*za(p9,b1)*zb(p1,b1)*zb(p2,a7)*zb(p4,q4)*zb(p10,u1
     &    ) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p9,b2)*zb(p1,p4)*zb(p2
     &    ,b2)*zb(p10,a7) - za(p1,p2)**2*za(p1,p9)*za(p2,b2)*za(p7,a7)*
     &    zb(p1,p4)*zb(p2,b2)*zb(p10,a7) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p9)*za(p7,a7)*zb(p1,p4)*zb(p10,a7) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p9,q4)*zb(p1,p10)*zb(
     &    p2,a7)*zb(p4,q4) - za(p1,p2)**2*za(p1,p7)*za(p2,u2)*za(p9,q4)
     &    *zb(p1,p10)*zb(p2,u2)*zb(p4,q4) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p7)*za(p9,q4)*zb(p1,p10)*zb(p4,q4) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p2,p9)*za(p7,a7)*zb(p1,p4)*zb(p2
     &    ,p10)*zb(p2,a7) - za(p1,p2)**2*za(p1,p7)*za(p2,q4)*za(p9,b1)*
     &    zb(p1,b1)*zb(p2,p10)*zb(p4,q4) - za(p1,p2)**2*za(p1,u1)*za(p2
     &    ,p9)*za(p7,a7)*zb(p1,p4)*zb(p2,a7)*zb(p10,u1) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-2)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**2*za(p1,p7)*za(p2,p9)*zb(p1,p4)*zb(p2
     &    ,p10) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)**2*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p2,a7)*zb(
     &    p4,q4)*zb(p10,u2) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p2)**2*za(p1,p9)*za(p7,a7)*zb(p1,p4)*
     &    zb(p10,a7) - za(p1,p2)*za(p1,p9)*za(p2,q4)*za(p7,a7)*zb(p4,q4
     &    )*zb(p10,a7) )
      mba(1,1,1,1) = mba(1,1,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p2)*za(p1,p7)*za(p2,u2)*za(p9,q4)*zb(p4
     &    ,q4)*zb(p10,u2) )

      mba(1,1,1,2)= + s12**(-2)*denb**(-1)*denq2**(-1) * ( za(p1,p2)**2
     &    *za(p1,b1)*za(p2,q4)*za(p7,a7)*za(p10,b2)*zb(p1,b1)*zb(p2,b2)
     &    *zb(p4,q4)*zb(p9,a7) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*denb**(-1) * ( za(p1,p2)
     &    **2*za(p1,q4)*za(p7,a7)*za(p10,b1)*zb(p1,b1)*zb(p4,q4)*zb(p9,
     &    a7) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,p2)**2*za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1
     &    ,u1)*zb(p2,a7)*zb(p4,q4)*zb(p9,u2) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*denu**(-1) * ( za(p1,p2)
     &    **2*za(p1,u1)*za(p7,a7)*za(p10,q4)*zb(p1,a7)*zb(p4,q4)*zb(p9,
     &    u1) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)**3*za(p2,q4)*za(p7,a7)*za(p10,b1)*zb(p1,b1)*
     &    zb(p2,p9)*zb(p2,a7)*zb(p4,q4) + za(p1,p2)**2*za(p1,u1)*za(p2,
     &    q4)*za(p7,a7)*za(p10,b1)*zb(p1,b1)*zb(p2,a7)*zb(p4,q4)*zb(p9,
     &    u1) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p10,b2)*zb(p1,p4)*zb(
     &    p2,b2)*zb(p9,a7) - za(p1,p2)**2*za(p1,p10)*za(p2,b2)*za(p7,a7
     &    )*zb(p1,p4)*zb(p2,b2)*zb(p9,a7) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p10)*za(p7,a7)*zb(p1,p4)*zb(p9,a7) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p7,a7)*za(p10,q4)*zb(p1,p9)*zb(
     &    p2,a7)*zb(p4,q4) - za(p1,p2)**2*za(p1,p7)*za(p2,u2)*za(p10,q4
     &    )*zb(p1,p9)*zb(p2,u2)*zb(p4,q4) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,p7)*za(p10,q4)*zb(p1,p9)*zb(p4,q4) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p2,p10)*za(p7,a7)*zb(p1,p4)*zb(
     &    p2,p9)*zb(p2,a7) - za(p1,p2)**2*za(p1,p7)*za(p2,q4)*za(p10,b1
     &    )*zb(p1,b1)*zb(p2,p9)*zb(p4,q4) - za(p1,p2)**2*za(p1,u1)*za(
     &    p2,p10)*za(p7,a7)*zb(p1,p4)*zb(p2,a7)*zb(p9,u1) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-2)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**2*za(p1,p7)*za(p2,p10)*zb(p1,p4)*zb(
     &    p2,p9) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)**2*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p2,a7)*
     &    zb(p4,q4)*zb(p9,u2) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p2)**2*za(p1,p10)*za(p7,a7)*zb(p1,p4)*
     &    zb(p9,a7) - za(p1,p2)*za(p1,p10)*za(p2,q4)*za(p7,a7)*zb(p4,q4
     &    )*zb(p9,a7) )
      mba(1,1,1,2) = mba(1,1,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p2)*za(p1,p7)*za(p2,u2)*za(p10,q4)*zb(
     &    p4,q4)*zb(p9,u2) )

      mba(1,1,2,1)= + s12**(-2)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p1,p2)**2*za(p1,b1)*za(p2,q4)*za(p7,p9)*zb(p1,b1)*zb(p2,p10)*
     &    zb(p4,q4) + za(p1,p2)**2*za(p1,b1)*za(p2,b2)*za(p7,p9)*zb(p1,
     &    p4)*zb(p2,b2)*zb(p10,b1) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,q4)*za(p7,p9)*zb(p1,p10)*zb(p4,q4) + za(p1
     &    ,p2)**2*za(p1,b1)*za(p7,p9)*zb(p1,p4)*zb(p10,b1) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p9,u1)*zb(p1,u1)*
     &    zb(p2,a7)*zb(p4,p10) + za(p1,p2)**2*za(p1,p7)*za(p2,u2)*za(p9
     &    ,u1)*zb(p1,u1)*zb(p2,u2)*zb(p4,p10) - za(p1,p2)**2*za(p1,u1)*
     &    za(p2,p9)*za(p7,a7)*zb(p1,u1)*zb(p2,a7)*zb(p4,p10) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p2)**2*za(p1,p7)*za(p9,u1)*zb(p1,u1)*zb(p4,p10) - za(p1,p2
     &    )**2*za(p1,p9)*za(p7,a7)*zb(p1,a7)*zb(p4,p10) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p1,p7)*za(p9,u2)*zb(p1,p4)*zb(p1
     &    ,p10)*zb(p2,u2) - za(p1,p2)**3*za(p1,p9)*za(p7,a7)*zb(p1,p4)*
     &    zb(p1,p10)*zb(p2,a7) + za(p1,p2)**2*za(p1,p7)*za(p2,q4)*za(p9
     &    ,u2)*zb(p1,p10)*zb(p2,u2)*zb(p4,q4) - za(p1,p2)**2*za(p1,p7)*
     &    za(p2,b2)*za(p9,u2)*zb(p1,p4)*zb(p2,u2)*zb(p10,b2) - za(p1,p2
     &    )**2*za(p1,p9)*za(p2,q4)*za(p7,a7)*zb(p1,p10)*zb(p2,a7)*zb(p4
     &    ,q4) + za(p1,p2)**2*za(p1,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p4)*
     &    zb(p2,a7)*zb(p10,b2) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-2)*mt**4*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p2)**3*za(p7,p9)*zb(p1,p4)*zb(p2,p10) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p1,p2)**2*za(p1,b1)*za(p7,p9)*zb(p1,p4)*zb(
     &    p10,b1) + za(p1,p2)*za(p1,b1)*za(p2,q4)*za(p7,p9)*zb(p4,q4)*
     &    zb(p10,b1) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)**2*za(p2,p9)*za(p7,a7)*zb(p2,a7)*zb(p4
     &    ,p10) )
      mba(1,1,2,1) = mba(1,1,2,1) + s12**(-1)*mt**4*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p1,p7)*za(p2,p9)*zb(p4,p10) )

      mba(1,1,2,2)= + s12**(-2)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p1,p2)**2*za(p1,b1)*za(p2,q4)*za(p7,p10)*zb(p1,b1)*zb(p2,p9)*
     &    zb(p4,q4) + za(p1,p2)**2*za(p1,b1)*za(p2,b2)*za(p7,p10)*zb(p1
     &    ,p4)*zb(p2,b2)*zb(p9,b1) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p2)**2*za(p1,q4)*za(p7,p10)*zb(p1,p9)*zb(p4,q4) + za(p1
     &    ,p2)**2*za(p1,b1)*za(p7,p10)*zb(p1,p4)*zb(p9,b1) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p2)**3*za(p7,a7)*za(p10,u1)*zb(p1,u1)*
     &    zb(p2,a7)*zb(p4,p9) + za(p1,p2)**2*za(p1,p7)*za(p2,u2)*za(p10
     &    ,u1)*zb(p1,u1)*zb(p2,u2)*zb(p4,p9) - za(p1,p2)**2*za(p1,u1)*
     &    za(p2,p10)*za(p7,a7)*zb(p1,u1)*zb(p2,a7)*zb(p4,p9) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p2)**2*za(p1,p7)*za(p10,u1)*zb(p1,u1)*zb(p4,p9) - za(p1,p2
     &    )**2*za(p1,p10)*za(p7,a7)*zb(p1,a7)*zb(p4,p9) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)**3*za(p1,p7)*za(p10,u2)*zb(p1,p4)*zb(
     &    p1,p9)*zb(p2,u2) - za(p1,p2)**3*za(p1,p10)*za(p7,a7)*zb(p1,p4
     &    )*zb(p1,p9)*zb(p2,a7) + za(p1,p2)**2*za(p1,p7)*za(p2,q4)*za(
     &    p10,u2)*zb(p1,p9)*zb(p2,u2)*zb(p4,q4) - za(p1,p2)**2*za(p1,p7
     &    )*za(p2,b2)*za(p10,u2)*zb(p1,p4)*zb(p2,u2)*zb(p9,b2) - za(p1,
     &    p2)**2*za(p1,p10)*za(p2,q4)*za(p7,a7)*zb(p1,p9)*zb(p2,a7)*zb(
     &    p4,q4) + za(p1,p2)**2*za(p1,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p4
     &    )*zb(p2,a7)*zb(p9,b2) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-2)*mt**4*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p2)**3*za(p7,p10)*zb(p1,p4)*zb(p2,p9) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p1,p2)**2*za(p1,b1)*za(p7,p10)*zb(p1,p4)*zb(
     &    p9,b1) + za(p1,p2)*za(p1,b1)*za(p2,q4)*za(p7,p10)*zb(p4,q4)*
     &    zb(p9,b1) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)**2*za(p2,p10)*za(p7,a7)*zb(p2,a7)*zb(
     &    p4,p9) )
      mba(1,1,2,2) = mba(1,1,2,2) + s12**(-1)*mt**4*denu**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p1,p7)*za(p2,p10)*zb(p4,p9) )

      mba(2,2,1,1)= + s12**(-2)*denb**(-1)*denq2**(-1) * ( za(p1,q4)*
     &    za(p2,b2)*za(p7,a7)*za(p9,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,b2
     &    )*zb(p4,q4)*zb(p10,a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*denb**(-1) * ( za(p1,q4)*
     &    za(p7,a7)*za(p9,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4,q4)*zb(p10,
     &    a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,
     &    a7)*zb(p2,u2)*zb(p4,q4)*zb(p10,u1) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*denu**(-1) * ( za(p1,u1)*
     &    za(p7,a7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,q4)*zb(p10,
     &    u1) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p9)*za(p1,q4)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**3*zb(
     &    p1,a7)*zb(p4,q4)*zb(p10,u2) + za(p1,q4)*za(p2,u2)*za(p7,a7)*
     &    za(p9,b2)*zb(p1,p2)**2*zb(p1,a7)*zb(p2,b2)*zb(p4,q4)*zb(p10,
     &    u2) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,q4)*za(p2,p9)*za(p7,a7)*zb(p1,p2)**3*
     &    zb(p4,q4)*zb(p10,a7) - za(p1,b1)*za(p2,p9)*za(p7,a7)*zb(p1,p2
     &    )**2*zb(p1,b1)*zb(p2,p4)*zb(p10,a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p9)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p10,a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,u1)*za(p2,p7)*za(p9,q4)*zb(p1,p2)**3*
     &    zb(p4,q4)*zb(p10,u1) - za(p1,u1)*za(p2,p7)*za(p9,q4)*zb(p1,p2
     &    )**2*zb(p1,u1)*zb(p2,p10)*zb(p4,q4) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p7)*za(p9,q4)*zb(p1,p2)**2*zb(p1,p10)*zb(p4,q4) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p9)*za(p1,q4)*za(p2,p7)*zb(p1,p2)**3*zb(p1
     &    ,p10)*zb(p4,q4) - za(p1,p9)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**2*
     &    zb(p1,a7)*zb(p2,p4)*zb(p10,u2) - za(p1,q4)*za(p2,p7)*za(p9,b2
     &    )*zb(p1,p2)**2*zb(p1,p10)*zb(p2,b2)*zb(p4,q4) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-2)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p9)*za(p2,p7)*zb(p1,p2)**2*zb(p1,p10)*zb(
     &    p2,p4) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-1)*denb**(-1)*denq2**(-1)
     &  * ( za(p1,q4)*za(p7,a7)*za(p9,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4,
     &    q4)*zb(p10,a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p7,a7)*za(p9,b1)*zb(p1,p2)*zb(p1,b1)*zb(p2,p4
     &    )*zb(p10,a7) )
      mba(2,2,1,1) = mba(2,2,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p9,q4)*zb(p1,p2)**2*zb(p2,p10)*zb(
     &    p4,q4) + za(p7,a7)*za(p9,q4)*zb(p1,p2)*zb(p1,a7)*zb(p2,p10)*
     &    zb(p4,q4) )

      mba(2,2,1,2)= + s12**(-2)*denb**(-1)*denq2**(-1) * ( za(p1,q4)*
     &    za(p2,b2)*za(p7,a7)*za(p10,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p2,
     &    b2)*zb(p4,q4)*zb(p9,a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*denb**(-1) * ( za(p1,q4)*
     &    za(p7,a7)*za(p10,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4,q4)*zb(p9,
     &    a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1,p2)**2*zb(p1
     &    ,a7)*zb(p2,u2)*zb(p4,q4)*zb(p9,u1) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*denu**(-1) * ( za(p1,u1)*
     &    za(p7,a7)*za(p10,q4)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,q4)*zb(p9,
     &    u1) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p10)*za(p1,q4)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**3*
     &    zb(p1,a7)*zb(p4,q4)*zb(p9,u2) + za(p1,q4)*za(p2,u2)*za(p7,a7)
     &    *za(p10,b2)*zb(p1,p2)**2*zb(p1,a7)*zb(p2,b2)*zb(p4,q4)*zb(p9,
     &    u2) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,q4)*za(p2,p10)*za(p7,a7)*zb(p1,p2)**3*
     &    zb(p4,q4)*zb(p9,a7) - za(p1,b1)*za(p2,p10)*za(p7,a7)*zb(p1,p2
     &    )**2*zb(p1,b1)*zb(p2,p4)*zb(p9,a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,p10)*za(p7,a7)*zb(p1,p2)**2*zb(p1,p4)*zb(p9,a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,u1)*za(p2,p7)*za(p10,q4)*zb(p1,p2)**3*
     &    zb(p4,q4)*zb(p9,u1) - za(p1,u1)*za(p2,p7)*za(p10,q4)*zb(p1,p2
     &    )**2*zb(p1,u1)*zb(p2,p9)*zb(p4,q4) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**2*denu**(-1) * (  - 
     &    za(p1,p7)*za(p10,q4)*zb(p1,p2)**2*zb(p1,p9)*zb(p4,q4) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p10)*za(p1,q4)*za(p2,p7)*zb(p1,p2)**3*zb(
     &    p1,p9)*zb(p4,q4) - za(p1,p10)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**
     &    2*zb(p1,a7)*zb(p2,p4)*zb(p9,u2) - za(p1,q4)*za(p2,p7)*za(p10,
     &    b2)*zb(p1,p2)**2*zb(p1,p9)*zb(p2,b2)*zb(p4,q4) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-2)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p10)*za(p2,p7)*zb(p1,p2)**2*zb(p1,p9)*zb(
     &    p2,p4) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-1)*denb**(-1)*denq2**(-1)
     &  * ( za(p1,q4)*za(p7,a7)*za(p10,b1)*zb(p1,p2)**2*zb(p1,b1)*zb(p4
     &    ,q4)*zb(p9,a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * ( za(p7,a7)*za(p10,b1)*zb(p1,p2)*zb(p1,b1)*zb(p2,
     &    p4)*zb(p9,a7) )
      mba(2,2,1,2) = mba(2,2,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p10,q4)*zb(p1,p2)**2*zb(p2,p9)*zb(
     &    p4,q4) + za(p7,a7)*za(p10,q4)*zb(p1,p2)*zb(p1,a7)*zb(p2,p9)*
     &    zb(p4,q4) )

      mba(2,2,2,1)= + s12**(-2)*mt**2*denb**(-1)*denq2**(-1) * ( za(p1,
     &    q4)*za(p2,b2)*za(p7,p9)*zb(p1,p2)**3*zb(p4,q4)*zb(p10,b2) - 
     &    za(p1,q4)*za(p2,b2)*za(p7,p9)*zb(p1,p2)**2*zb(p1,p10)*zb(p2,
     &    b2)*zb(p4,q4) + za(p1,b1)*za(p2,b2)*za(p7,p9)*zb(p1,p2)**2*
     &    zb(p1,b1)*zb(p2,p4)*zb(p10,b2) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,q4)*za(p7,p9)*zb(p1,p2)**2*zb(p1,p10)*zb(p4,q4) + za(p1
     &    ,b1)*za(p7,p9)*zb(p1,p2)**2*zb(p1,p4)*zb(p10,b1) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p9)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**2*
     &    zb(p1,a7)*zb(p2,u2)*zb(p4,p10) + za(p1,u1)*za(p2,p7)*za(p9,u2
     &    )*zb(p1,p2)**2*zb(p1,u1)*zb(p2,u2)*zb(p4,p10) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p7)*za(p9,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p4,p10) - za(p1,p9
     &    )*za(p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,p10) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * (  - za(p1,q4)*za(p2,p7)*za(p2,p9)*zb(p1,p2)**3*
     &    zb(p2,p10)*zb(p4,q4) + za(p1,q4)*za(p2,p7)*za(p9,u1)*zb(p1,p2
     &    )**2*zb(p1,u1)*zb(p2,p10)*zb(p4,q4) - za(p1,q4)*za(p2,p9)*za(
     &    p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p2,p10)*zb(p4,q4) + za(p1,b1
     &    )*za(p2,p7)*za(p2,p9)*zb(p1,p2)**3*zb(p2,p4)*zb(p10,b1) - za(
     &    p1,b1)*za(p2,p7)*za(p9,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p2,p4)*
     &    zb(p10,b1) + za(p1,b1)*za(p2,p9)*za(p7,a7)*zb(p1,p2)**2*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p10,b1) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-2)*mt**4*denu**(-1)*
     & dena1**(-1) * ( za(p1,p9)*za(p2,p7)*zb(p1,p2)**3*zb(p4,p10) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,q4)*za(p7,p9)*zb(p1,p2)**2*zb(p1,p10)*
     &    zb(p4,q4) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p9,u2)*zb(p1,p2)**2*zb(p2,u2)*
     &    zb(p4,p10) - za(p7,a7)*za(p9,u2)*zb(p1,p2)*zb(p1,a7)*zb(p2,u2
     &    )*zb(p4,p10) )
      mba(2,2,2,1) = mba(2,2,2,1) + s12**(-1)*mt**4*denb**(-1)*
     & denq2**(-1) * (  - za(p7,p9)*zb(p1,p2)*zb(p1,p10)*zb(p2,p4) )

      mba(2,2,2,2)= + s12**(-2)*mt**2*denb**(-1)*denq2**(-1) * ( za(p1,
     &    q4)*za(p2,b2)*za(p7,p10)*zb(p1,p2)**3*zb(p4,q4)*zb(p9,b2) - 
     &    za(p1,q4)*za(p2,b2)*za(p7,p10)*zb(p1,p2)**2*zb(p1,p9)*zb(p2,
     &    b2)*zb(p4,q4) + za(p1,b1)*za(p2,b2)*za(p7,p10)*zb(p1,p2)**2*
     &    zb(p1,b1)*zb(p2,p4)*zb(p9,b2) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-2)*mt**2*denb**(-1) * (  - 
     &    za(p1,q4)*za(p7,p10)*zb(p1,p2)**2*zb(p1,p9)*zb(p4,q4) + za(p1
     &    ,b1)*za(p7,p10)*zb(p1,p2)**2*zb(p1,p4)*zb(p9,b1) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-2)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p10)*za(p2,u2)*za(p7,a7)*zb(p1,p2)**2*
     &    zb(p1,a7)*zb(p2,u2)*zb(p4,p9) + za(p1,u1)*za(p2,p7)*za(p10,u2
     &    )*zb(p1,p2)**2*zb(p1,u1)*zb(p2,u2)*zb(p4,p9) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-2)*mt**2*denu**(-1) * ( za(
     &    p1,p7)*za(p10,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p4,p9) - za(p1,
     &    p10)*za(p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p4,p9) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-2)*mt**2*denq2**(-1)*
     & dena1**(-1) * (  - za(p1,q4)*za(p2,p7)*za(p2,p10)*zb(p1,p2)**3*
     &    zb(p2,p9)*zb(p4,q4) + za(p1,q4)*za(p2,p7)*za(p10,u1)*zb(p1,p2
     &    )**2*zb(p1,u1)*zb(p2,p9)*zb(p4,q4) - za(p1,q4)*za(p2,p10)*za(
     &    p7,a7)*zb(p1,p2)**2*zb(p1,a7)*zb(p2,p9)*zb(p4,q4) + za(p1,b1)
     &    *za(p2,p7)*za(p2,p10)*zb(p1,p2)**3*zb(p2,p4)*zb(p9,b1) - za(
     &    p1,b1)*za(p2,p7)*za(p10,u1)*zb(p1,p2)**2*zb(p1,u1)*zb(p2,p4)*
     &    zb(p9,b1) + za(p1,b1)*za(p2,p10)*za(p7,a7)*zb(p1,p2)**2*zb(p1
     &    ,a7)*zb(p2,p4)*zb(p9,b1) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-2)*mt**4*denu**(-1)*
     & dena1**(-1) * ( za(p1,p10)*za(p2,p7)*zb(p1,p2)**3*zb(p4,p9) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,q4)*za(p7,p10)*zb(p1,p2)**2*zb(p1,p9)*
     &    zb(p4,q4) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p10,u2)*zb(p1,p2)**2*zb(p2,u2)*
     &    zb(p4,p9) - za(p7,a7)*za(p10,u2)*zb(p1,p2)*zb(p1,a7)*zb(p2,u2
     &    )*zb(p4,p9) )
      mba(2,2,2,2) = mba(2,2,2,2) + s12**(-1)*mt**4*denb**(-1)*
     & denq2**(-1) * (  - za(p7,p10)*zb(p1,p2)*zb(p1,p9)*zb(p2,p4) )

      mba(1,2,1,1)= + s12**(-1)*denb**(-1)*denq2**(-1) * ( za(p1,q4)*
     &    za(p1,b2)*za(p7,a7)*za(p9,b2)*zb(p2,b2)**2*zb(p4,q4)*zb(p10,
     &    a7) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,u1)*za(p1,u2)*za(p7,a7)*za(p9,q4)*zb(p2,a7)*zb(p2,u2)
     &    *zb(p4,q4)*zb(p10,u1) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)*za(p1,q4)*za(p7,a7)*za(p9,b2)*zb(p2,p10)*zb(p2
     &    ,a7)*zb(p2,b2)*zb(p4,q4) - za(p1,p9)*za(p1,q4)*za(p1,u1)*za(
     &    p7,a7)*zb(p1,p2)*zb(p2,a7)*zb(p4,q4)*zb(p10,u1) + za(p1,q4)*
     &    za(p1,u1)*za(p7,a7)*za(p9,b2)*zb(p2,a7)*zb(p2,b2)*zb(p4,q4)*
     &    zb(p10,u1) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p9)*za(p1,b1)*za(p7,a7)*zb(p2,p4)*zb(p2
     &    ,b1)*zb(p10,a7) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p7)*za(p1,u1)*za(p9,q4)*zb(p2,p10)*zb(
     &    p2,u1)*zb(p4,q4) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p1,p9)*za(p7,a7)*zb(p2,p4)*zb(p2,
     &    p10)*zb(p2,a7) + za(p1,p7)*za(p1,p9)*za(p1,q4)*zb(p1,p2)*zb(
     &    p2,p10)*zb(p4,q4) - za(p1,p7)*za(p1,q4)*za(p9,b2)*zb(p2,p10)*
     &    zb(p2,b2)*zb(p4,q4) - za(p1,p9)*za(p1,u1)*za(p7,a7)*zb(p2,p4)
     &    *zb(p2,a7)*zb(p10,u1) )
      mba(1,2,1,1) = mba(1,2,1,1) + s12**(-1)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,p9)*zb(p2,p4)*zb(p2,p10) )
      mba(1,2,1,1) = mba(1,2,1,1) + denq2**(-1)*dena1**(-1) * (  - za(
     &    p1,p9)*za(p1,q4)*za(p7,a7)*zb(p2,p10)*zb(p2,a7)*zb(p4,q4) )

      mba(1,2,1,2)= + s12**(-1)*denb**(-1)*denq2**(-1) * ( za(p1,q4)*
     &    za(p1,b2)*za(p7,a7)*za(p10,b2)*zb(p2,b2)**2*zb(p4,q4)*zb(p9,
     &    a7) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * ( za(p1,u1)*za(p1,u2)*za(p7,a7)*za(p10,q4)*zb(p2,a7)*zb(p2,u2
     &    )*zb(p4,q4)*zb(p9,u1) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*denq2**(-1)*dena1**(-1)
     &  * (  - za(p1,p2)*za(p1,q4)*za(p7,a7)*za(p10,b2)*zb(p2,p9)*zb(p2
     &    ,a7)*zb(p2,b2)*zb(p4,q4) - za(p1,p10)*za(p1,q4)*za(p1,u1)*za(
     &    p7,a7)*zb(p1,p2)*zb(p2,a7)*zb(p4,q4)*zb(p9,u1) + za(p1,q4)*
     &    za(p1,u1)*za(p7,a7)*za(p10,b2)*zb(p2,a7)*zb(p2,b2)*zb(p4,q4)*
     &    zb(p9,u1) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p1,p10)*za(p1,b1)*za(p7,a7)*zb(p2,p4)*zb(
     &    p2,b1)*zb(p9,a7) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p1,p7)*za(p1,u1)*za(p10,q4)*zb(p2,p9)*zb(
     &    p2,u1)*zb(p4,q4) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p1,p10)*za(p7,a7)*zb(p2,p4)*zb(p2,
     &    p9)*zb(p2,a7) + za(p1,p7)*za(p1,p10)*za(p1,q4)*zb(p1,p2)*zb(
     &    p2,p9)*zb(p4,q4) - za(p1,p7)*za(p1,q4)*za(p10,b2)*zb(p2,p9)*
     &    zb(p2,b2)*zb(p4,q4) - za(p1,p10)*za(p1,u1)*za(p7,a7)*zb(p2,p4
     &    )*zb(p2,a7)*zb(p9,u1) )
      mba(1,2,1,2) = mba(1,2,1,2) + s12**(-1)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,p10)*zb(p2,p4)*zb(p2,p9) )
      mba(1,2,1,2) = mba(1,2,1,2) + denq2**(-1)*dena1**(-1) * (  - za(
     &    p1,p10)*za(p1,q4)*za(p7,a7)*zb(p2,p9)*zb(p2,a7)*zb(p4,q4) )

      mba(1,2,2,1)= + s12**(-1)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p1,q4)*za(p1,b2)*za(p7,p9)*zb(p2,p10)*zb(p2,b2)*zb(p4,q4) + 
     &    za(p1,b1)**2*za(p7,p9)*zb(p2,p4)*zb(p2,b1)*zb(p10,b1) )
      mba(1,2,2,1) = mba(1,2,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,u1)*za(p9,u2)*zb(p2,u1)*zb(p2,u2
     &    )*zb(p4,p10) - za(p1,p9)*za(p1,u2)*za(p7,a7)*zb(p2,a7)*zb(p2,
     &    u2)*zb(p4,p10) )
      mba(1,2,2,1) = mba(1,2,2,1) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,q4)*za(p9,u2)*zb(p2,p10)*zb(p2,
     &    u2)*zb(p4,q4) - za(p1,p7)*za(p1,b1)*za(p9,u2)*zb(p2,p4)*zb(p2
     &    ,u2)*zb(p10,b1) - za(p1,p9)*za(p1,q4)*za(p7,a7)*zb(p2,p10)*
     &    zb(p2,a7)*zb(p4,q4) + za(p1,p9)*za(p1,b1)*za(p7,a7)*zb(p2,p4)
     &    *zb(p2,a7)*zb(p10,b1) )

      mba(1,2,2,2)= + s12**(-1)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p1,q4)*za(p1,b2)*za(p7,p10)*zb(p2,p9)*zb(p2,b2)*zb(p4,q4) + 
     &    za(p1,b1)**2*za(p7,p10)*zb(p2,p4)*zb(p2,b1)*zb(p9,b1) )
      mba(1,2,2,2) = mba(1,2,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,u1)*za(p10,u2)*zb(p2,u1)*zb(p2,
     &    u2)*zb(p4,p9) - za(p1,p10)*za(p1,u2)*za(p7,a7)*zb(p2,a7)*zb(
     &    p2,u2)*zb(p4,p9) )
      mba(1,2,2,2) = mba(1,2,2,2) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p7)*za(p1,q4)*za(p10,u2)*zb(p2,p9)*zb(p2,
     &    u2)*zb(p4,q4) - za(p1,p7)*za(p1,b1)*za(p10,u2)*zb(p2,p4)*zb(
     &    p2,u2)*zb(p9,b1) - za(p1,p10)*za(p1,q4)*za(p7,a7)*zb(p2,p9)*
     &    zb(p2,a7)*zb(p4,q4) + za(p1,p10)*za(p1,b1)*za(p7,a7)*zb(p2,p4
     &    )*zb(p2,a7)*zb(p9,b1) )

      mba(2,1,1,1)= + s12**(-1)*denb**(-1)*denq2**(-1) * ( za(p2,q4)*
     &    za(p2,b1)*za(p7,a7)*za(p9,b1)*zb(p1,b1)**2*zb(p4,q4)*zb(p10,
     &    a7) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * ( za(p2,u1)*za(p2,u2)*za(p7,a7)*za(p9,q4)*zb(p1,a7)*zb(p1,u1)
     &    *zb(p4,q4)*zb(p10,u2) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*denq2**(-1)*dena1**(-1)
     &  * ( za(p2,q4)*za(p2,u2)*za(p7,a7)*za(p9,b1)*zb(p1,a7)*zb(p1,b1)
     &    *zb(p4,q4)*zb(p10,u2) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p2,p9)*za(p2,b2)*za(p7,a7)*zb(p1,p4)*zb(p1
     &    ,b2)*zb(p10,a7) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p2,u2)*za(p9,q4)*zb(p1,p10)*zb(
     &    p1,u2)*zb(p4,q4) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p2,q4)*za(p9,b1)*zb(p1,p10)*zb(
     &    p1,b1)*zb(p4,q4) - za(p2,p9)*za(p2,u2)*za(p7,a7)*zb(p1,p4)*
     &    zb(p1,a7)*zb(p10,u2) )
      mba(2,1,1,1) = mba(2,1,1,1) + s12**(-1)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p2,p9)*zb(p1,p4)*zb(p1,p10) )

      mba(2,1,1,2)= + s12**(-1)*denb**(-1)*denq2**(-1) * ( za(p2,q4)*
     &    za(p2,b1)*za(p7,a7)*za(p10,b1)*zb(p1,b1)**2*zb(p4,q4)*zb(p9,
     &    a7) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*denu**(-1)*dena1**(-1)
     &  * ( za(p2,u1)*za(p2,u2)*za(p7,a7)*za(p10,q4)*zb(p1,a7)*zb(p1,u1
     &    )*zb(p4,q4)*zb(p9,u2) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*denq2**(-1)*dena1**(-1)
     &  * ( za(p2,q4)*za(p2,u2)*za(p7,a7)*za(p10,b1)*zb(p1,a7)*zb(p1,b1
     &    )*zb(p4,q4)*zb(p9,u2) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*mt**2*denb**(-1)*
     & denq2**(-1) * (  - za(p2,p10)*za(p2,b2)*za(p7,a7)*zb(p1,p4)*zb(
     &    p1,b2)*zb(p9,a7) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p2,u2)*za(p10,q4)*zb(p1,p9)*zb(
     &    p1,u2)*zb(p4,q4) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * (  - za(p2,p7)*za(p2,q4)*za(p10,b1)*zb(p1,p9)*zb(
     &    p1,b1)*zb(p4,q4) - za(p2,p10)*za(p2,u2)*za(p7,a7)*zb(p1,p4)*
     &    zb(p1,a7)*zb(p9,u2) )
      mba(2,1,1,2) = mba(2,1,1,2) + s12**(-1)*mt**4*denq2**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p2,p10)*zb(p1,p4)*zb(p1,p9) )

      mba(2,1,2,1)= + s12**(-1)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p2,q4)*za(p2,b1)*za(p7,p9)*zb(p1,p10)*zb(p1,b1)*zb(p4,q4) + 
     &    za(p2,b2)**2*za(p7,p9)*zb(p1,p4)*zb(p1,b2)*zb(p10,b2) )
      mba(2,1,2,1) = mba(2,1,2,1) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p2,u2)*za(p9,u1)*zb(p1,u1)*zb(p1,u2
     &    )*zb(p4,p10) - za(p2,p9)*za(p2,u1)*za(p7,a7)*zb(p1,a7)*zb(p1,
     &    u1)*zb(p4,p10) )
      mba(2,1,2,1) = mba(2,1,2,1) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p2,p7)*za(p9,u1)*zb(p1,p4)*zb(p1,
     &    p10)*zb(p1,u1) - za(p1,p2)*za(p2,p9)*za(p7,a7)*zb(p1,p4)*zb(
     &    p1,p10)*zb(p1,a7) - za(p2,p7)*za(p2,p9)*za(p2,q4)*zb(p1,p2)*
     &    zb(p1,p10)*zb(p4,q4) + za(p2,p7)*za(p2,p9)*za(p2,b2)*zb(p1,p2
     &    )*zb(p1,p4)*zb(p10,b2) + za(p2,p7)*za(p2,q4)*za(p9,u1)*zb(p1,
     &    p10)*zb(p1,u1)*zb(p4,q4) - za(p2,p7)*za(p2,b2)*za(p9,u1)*zb(
     &    p1,p4)*zb(p1,u1)*zb(p10,b2) - za(p2,p9)*za(p2,q4)*za(p7,a7)*
     &    zb(p1,p10)*zb(p1,a7)*zb(p4,q4) + za(p2,p9)*za(p2,b2)*za(p7,a7
     &    )*zb(p1,p4)*zb(p1,a7)*zb(p10,b2) )
      mba(2,1,2,1) = mba(2,1,2,1) + mt**2*denq2**(-1)*dena1**(-1) * ( 
     &    za(p2,p7)*za(p2,p9)*zb(p1,p4)*zb(p1,p10) )

      mba(2,1,2,2)= + s12**(-1)*mt**2*denb**(-1)*denq2**(-1) * (  - za(
     &    p2,q4)*za(p2,b1)*za(p7,p10)*zb(p1,p9)*zb(p1,b1)*zb(p4,q4) + 
     &    za(p2,b2)**2*za(p7,p10)*zb(p1,p4)*zb(p1,b2)*zb(p9,b2) )
      mba(2,1,2,2) = mba(2,1,2,2) + s12**(-1)*mt**2*denu**(-1)*
     & dena1**(-1) * ( za(p2,p7)*za(p2,u2)*za(p10,u1)*zb(p1,u1)*zb(p1,
     &    u2)*zb(p4,p9) - za(p2,p10)*za(p2,u1)*za(p7,a7)*zb(p1,a7)*zb(
     &    p1,u1)*zb(p4,p9) )
      mba(2,1,2,2) = mba(2,1,2,2) + s12**(-1)*mt**2*denq2**(-1)*
     & dena1**(-1) * ( za(p1,p2)*za(p2,p7)*za(p10,u1)*zb(p1,p4)*zb(p1,
     &    p9)*zb(p1,u1) - za(p1,p2)*za(p2,p10)*za(p7,a7)*zb(p1,p4)*zb(
     &    p1,p9)*zb(p1,a7) - za(p2,p7)*za(p2,p10)*za(p2,q4)*zb(p1,p2)*
     &    zb(p1,p9)*zb(p4,q4) + za(p2,p7)*za(p2,p10)*za(p2,b2)*zb(p1,p2
     &    )*zb(p1,p4)*zb(p9,b2) + za(p2,p7)*za(p2,q4)*za(p10,u1)*zb(p1,
     &    p9)*zb(p1,u1)*zb(p4,q4) - za(p2,p7)*za(p2,b2)*za(p10,u1)*zb(
     &    p1,p4)*zb(p1,u1)*zb(p9,b2) - za(p2,p10)*za(p2,q4)*za(p7,a7)*
     &    zb(p1,p9)*zb(p1,a7)*zb(p4,q4) + za(p2,p10)*za(p2,b2)*za(p7,a7
     &    )*zb(p1,p4)*zb(p1,a7)*zb(p9,b2) )
      mba(2,1,2,2) = mba(2,1,2,2) + mt**2*denq2**(-1)*dena1**(-1) * ( 
     &    za(p2,p7)*za(p2,p10)*zb(p1,p4)*zb(p1,p9) )

c--- sum over couplings of Z to top quark line
c--- (for charge and couplings of top quark, use up-charge)
      do h1=1,2
      do h2=1,2
      xab(h1,h2,1)=(Q(2)*q1+L(2)*l1*propz)*mab(h1,h2,1,1)
     &            +(Q(2)*q1+R(2)*l1*propz)*mab(h1,h2,2,1)
      xab(h1,h2,2)=(Q(2)*q1+L(2)*r1*propz)*mab(h1,h2,1,2)
     &            +(Q(2)*q1+R(2)*r1*propz)*mab(h1,h2,2,2)
      xba(h1,h2,1)=(Q(2)*q1+L(2)*l1*propz)*mba(h1,h2,1,1)
     &            +(Q(2)*q1+R(2)*l1*propz)*mba(h1,h2,2,1)
      xba(h1,h2,2)=(Q(2)*q1+L(2)*r1*propz)*mba(h1,h2,1,2)
     &            +(Q(2)*q1+R(2)*r1*propz)*mba(h1,h2,2,2)
      enddo
      enddo

      wtgg=0d0
      do h1=2,1,-1                               
      do h2=2,1,-1                               
      do h3=1,2                               
      wtgg=wtgg                               
     & +abs(xab(h1,h2,h3))**2                   
     & +abs(xba(h1,h2,h3))**2                   
     & -1d0/xnsq*abs(xab(h1,h2,h3)+xba(h1,h2,h3))**2 
      enddo
      enddo
      enddo

      return
      end
