!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine tdecayg(p,pqq,pqb,pb,pg,m)
      implicit none
      include 'src/Inc/types.f'
      
************************************************************************
*     Author: R.K. Ellis, January 2012                                 *
*     top decay  t --> q(pqq)+qb(pb)+b(pb)+g(pg)                       *
*     with bottom and top masses (and radiation form t-b line)         *
*     in massless spinor notation                                      *
*     pqq,pqb,pb,pg are integer::s that point to                         *
*     the appropriate four-momenta in p                                *
*     pqq=quark                                                        *
*     pqb=antiquark                                                    *
*     pb=bottom quark                                                  *
*     pg=gluon                                                         *
*     q(t) is rendered massless wrt to pqb                             *
*     q(b) is rendered massless wrt to pqq                             *
*     returned m(bpol,gpol,tpol)                                       *
************************************************************************
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/masses.f'
      real(dp):: p(mxpart,4),q(mxpart,4),dot,sw,twopbDg,twoptDg,
     & altg,alb
      complex(dp):: m(2,2,2),cprop
      integer:: qq,qb,b,g,tg,nu,pb,pqq,pqb,pg
      parameter(tg=1,g=2,qq=3,qb=4,b=5)
      do nu=1,4
      q(tg,nu)=p(pqq,nu)+p(pqb,nu)+p(pb,nu)+p(pg,nu)
      q(g,nu)=p(pg,nu)
      q(qq,nu)=p(pqq,nu)
      q(qb,nu)=p(pqb,nu)
      q(b,nu)=p(pb,nu)
      enddo
      twoptDg=2._dp*dot(q,tg,g)
      twopbDg=2._dp*dot(q,b,g)
      altg=mt**2/(2._dp*dot(q,tg,qb))
      alb=mb**2/(2._dp*dot(q,b,qq))
      do nu=1,4
      q(tg,nu)=q(tg,nu)-altg*q(qb,nu)
      q(b,nu)=q(b,nu)-alb*q(qq,nu)
      enddo
      call spinoru(5,q,za,zb)
      sw=s(qq,qb)
      cprop=cplx2(sw-wmass**2,wmass*wwidth)
C---order of polarizations is the m(bpol,gpol,tpol)
      m(1,1,1)= + cprop**(-1)*twoptDg**(-1) * ( za(qq,b)*za(g,tg)*zb(qb
     &    ,tg)*zb(b,tg)/zb(b,g) )
      m(1,1,1) = m(1,1,1) + cprop**(-1)*twopbDg**(-1) * ( za(qq,g)*za(b
     &    ,g)*zb(qb,tg) )
      m(1,1,1) = m(1,1,1) + cprop**(-1)*twopbDg**(-1)*mb**2 * ( za(qq,g
     &    )*zb(qb,tg)/zb(b,g) )
      m(1,1,1) = m(1,1,1) + cprop**(-1)*mt**2*twoptDg**(-1) * (  - za(
     &    qq,b)*za(qb,g)*zb(qb,b)/za(qb,tg)/zb(b,g) )

      m(1,2,1)= + cprop**(-1)*twoptDg**(-1) * (  - za(qq,b)*za(b,tg)*
     &    zb(qb,tg)*zb(g,tg)/za(b,g) + za(qq,b)*zb(qb,g)*zb(g,tg) )
      m(1,2,1) = m(1,2,1) + cprop**(-1)*twopbDg**(-1)*mb**2 * (  - za(
     &    qq,b)*zb(qq,g)*zb(qb,tg)/za(b,g)/zb(qq,b) )
      m(1,2,1) = m(1,2,1) + cprop**(-1)*mt**2*twoptDg**(-1) * ( za(qq,b
     &    )*za(qb,b)*zb(qb,g)/za(qb,tg)/za(b,g) )

      m(2,1,1)= + cprop**(-1)*twopbDg**(-1)*mb * ( za(qq,g)**2*zb(qb,tg
     &    )/za(qq,b) )

      m(2,2,1)= czip

      m(1,1,2)= czip

      m(1,2,2)= + cprop**(-1)*mt*twoptDg**(-1) * ( za(qq,b)*zb(qb,g)**2
     &    /zb(qb,tg) )

      m(2,1,2)= czip

      m(2,2,2)= czip

      return
      end
