!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 

      function a6treeQLlc(st,j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: a6treeQLlc

c-----Tree amplitude for leading color contributions
c----- 0 -> q(p1) + qb(p2) + glu(p3) + gam(p4) + lb(p5) + l(p6)
c-----where the photon is coming from the lepton line
      
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zprods_decl.f'
      integer:: st
      integer:: j1,j2,j3,j4,j5,j6
      real(dp):: t

c---- helicity stamps
c     'q+qb-g+ga+lb-l+'=4
c     'q+qb-g+ga-lb-l+'=5

c-----Multiplied by (-i), factor out sqrt(2) 
      if(st==4) then
      a6treeQLlc=
     .za(j2,j5)**2/(za(j1,j3)*za(j2,j3)*za(j4,j5)*za(j4,j6))
      elseif(st==5) then
      a6treeQLlc=
     .(za(j2,j4)*zb(j4,j6)+za(j2,j5)*zb(j5,j6))**2/
     .(t(j4,j5,j6)*za(j1,j3)*za(j2,j3)*zb(j4,j5)*zb(j4,j6))
      else
      write(6,*) 'unimplemented st'
      stop
      endif
c-----done
      return
      end


      function a6treeQLslc(st,j1,j2,j3,j4,j5,j6,za,zb)
      implicit none
      include 'src/Inc/types.f'
      complex(dp):: a6treeQLslc

c-----Tree amplitude for subleading color contributions
c----- 0 -> q(p1) + qb(p2) + glu(p3) + gam(p4) + lb(p5) + l(p6)
c-----where the photon is coming from  the lepton line
      
      integer:: j1,j2,j3,j4,j5,j6
      include 'src/Inc/constants.f'
      include 'src/Inc/nf.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/cplx.h'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer:: st
      real(dp):: t

c---- helicity stamps
c     'q+qb-g+ga+lb-l+'=4
c     'q+qb-g+ga-lb-l+'=5

c-----Multiplied by (-i), factor out sqrt(2) 
      if(st==4) then
      a6treeQLslc=
     .-za(j2,j5)**2/(za(j1,j3)*za(j2,j3)*za(j4,j5)*za(j4,j6))
      elseif(st==5) then
      a6treeQLslc= 
     .-(za(j2,j4)*zb(j4,j6)+za(j2,j5)*zb(j5,j6))**2/
     & (t(j4,j5,j6)*za(j1,j3)*za(j2,j3)*zb(j4,j5)*zb(j4,j6))
      else 
      write(6,*) 'unimplemented st'
      stop
      endif
      return
      end



