!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function aqmpB12(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.10) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: aqmpB12, aqmpB12_unsym

      aqmpB12 = (+aqmpB12_unsym(p1,p2,p3,p4,za,zb)
     &           -aqmpB12_unsym(p2,p1,p4,p3,zb,za))
      return
      end


      function aqmpB12_unsym(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^- 4_g^+ +H
!     Implementation of Eq.~(9.10) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s1234,lambda,lam
      complex(dp):: aqmpB12_unsym,zab2

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      lambda(s12,s13,s14)=(s12-s13-s14)**2-4*s13*s14                          

      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s1234=s12+s13+s14+s23+s24+s34
      lam=lambda(s1234,s12,s34)
      aqmpB12_unsym=4._dp/zab2(p4,p1,p2,p3)**2
     & *(
     & -za(p1,p2)*zb(p1,p3)**2*zab2(p3,p1,p2,p4)/(zb(p3,p4)*(s13+s23))
     & +zab2(p3,p1,p2,p4)
     & *(
     &  s12*za(p2,p4)*zb(p1,p3)*za(p3,p4)
     & -2*za(p1,p2)*zb(p1,p3)**2*za(p3,p4)**2
     & -za(p2,p4)**2*zb(p1,p2)*(s13+s23+s14+s24)
     & -za(p2,p3)*zab2(p4,p1,p2,p3)*zab2(p4,p2,p3,p1))
     & /(za(p3,p4)*lam))
     & -12*(s13+s23)*(2*s12+s13+s23)*zab2(p2,p3,p4,p1)*zab2(p3,p1,p2,p4)
     & /(zab2(p4,p1,p2,p3)*lam**2) 

      return
      end
