!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function aqppC3x412(p1,p2,p3,p4,za,zb,msqCoeff)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^+ 4_g^+ +H
!     Implementation of Eq.~(8.9) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp):: aqppC3x412,aqppC3x412m0,aqppC3x412m2,msqCoeff
      msqCoeff=aqppC3x412m2(p1,p2,p3,p4,za,zb)
      aqppC3x412=aqppC3x412m0(p1,p2,p3,p4,za,zb)+mtsq*msqCoeff
      return
      end

      function aqppC3x412m0(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^+ 4_g^+ +H
!     Implementation of Eq.~(8.9) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s124,s13p23p34,Tr3x12x4x12
      complex(dp):: aqppC3x412m0,zab2
 
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)

      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
                          
      s124=s12+s14+s24
      Tr3x12x4x12=(s13+s23)*(s14+s24)-s12*s34
      s13p23p34=s13+s23+s34

      aqppC3x412m0=
     & +2*zab2(p2,p1,p4,p3)**2*zab2(p1,p2,p4,p3)
     & /(za(p2,p1)*za(p1,p4)*zab2(p4,p1,p2,p3)*s124) 
     & +zb(p1,p4)*zb(p1,p3)*zb(p4,p3)*za(p4,p3)*s13p23p34
     & /(zb(p2,p1)*za(p4,p3)*Tr3x12x4x12)
     & +za(p2,p4)*za(p2,p3)*zb(p4,p3)**2*s13p23p34
     & /(za(p2,p1)*za(p4,p3)*Tr3x12x4x12)
     & -zb(p1,p4)**2*s13p23p34
     & /(zb(p2,p1)*za(p4,p3)*zab2(p3,p1,p2,p4))
     & -zb(p1,p3)**2*s13p23p34
     & /(zb(p2,p1)*za(p4,p3)*zab2(p4,p1,p2,p3))
     & -2*za(p2,p4)*za(p2,p3)*s13p23p34
     & /(za(p2,p1)*za(p4,p3)**3)
      return
      end


      function aqppC3x412m2(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_qb^+ 2_q^- 3_g^+ 4_g^+ +H
!     Implementation of Eq.~(8.10) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/sprods_com.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s124,s13p23p34,Tr3x12x4x12
      complex(dp):: aqppC3x412m2,zab2
 
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)

      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s124=s12+s14+s24
      s13p23p34=s13+s23+s34
      Tr3x12x4x12=(s13+s23)*(s14+s24)-s12*s34

      aqppC3x412m2=
     & -8*zab2(p2,p1,p4,p3)**2*zab2(p1,p2,p4,p3)
     & /(za(p2,p1)*za(p1,p4)*zab2(p4,p1,p2,p3)*s13p23p34*s124) 
     & +4*za(p2,p4)*za(p2,p3)*zb(p4,p3)*s13p23p34
     & /(za(p2,p1)*za(p4,p3)**2*Tr3x12x4x12) 
     & +4*zb(p1,p4)*zb(p1,p3)*s13p23p34
     & /(zb(p2,p1)*za(p4,p3)*Tr3x12x4x12)
      return
      end

