!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      subroutine fillformfactor(p1,p2,p3,p4,FL,FT)
      use mod_qcdloop_c
      use iso_c_binding
      use iso_fortran_env
!@article{DelDuca:2001fn,
!      author         = "Del Duca, V. and Kilgore, W. and Oleari, C. and Schmidt,
!                        C. and Zeppenfeld, D.",
!      title          = "{Gluon fusion contributions to H + 2 jet production}",
!      journal        = "Nucl. Phys.",
!      volume         = "B616",
!      year           = "2001",
!      pages          = "367-399",
!      doi            = "10.1016/S0550-3213(01)00446-1",
!      eprint         = "hep-ph/0108030",
!      archivePrefix  = "arXiv",
!      primaryClass   = "hep-ph",
!      reportNumber   = "MADPH-01-1235, BNL-HET-01-28, MSUHEP-10709, DFTT-19-2001",
!      SLACcitation   = "%%CITATION = HEP-PH/0108030;%%"
!}
!  Formula taken from DelDuca:2001fn Eqs.C7-C10 with modified normalization.
!     Implementation of Eq.~(11.4) and (11.5) from arXiv:2002.04018 v2
      implicit none
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/sprods_com.f'
      include 'src/ggHgg_mass/mtsq.f'
      complex(dp)::FL,FT,B0k1,B0k2,B0k12,C0k1xk2,xlI3,xlI2
      integer p1,p2,p3,p4
      real(dp)::k1Dk1,k2Dk2,k1Dk2,k1Dk12,k2Dk12,k12Dk12,Gram,msq,mu2
      real(real128)::msq_128,mu2_128
      msq=mtsq
      mu2=mtsq
      k1Dk1=s(p1,p2)
      k2Dk2=s(p3,p4)
      k1Dk2=0.5_dp*(s(p1,p3)+s(p1,p4)+s(p2,p3)+s(p2,p4))
      k1Dk12=k1Dk1+k1Dk2
      k2Dk12=k2Dk2+k1Dk2
      k12Dk12=k1Dk12+k2Dk12 
      C0k1xk2=qlI3(k1Dk1,k12Dk12,k2Dk2,msq,msq,msq,mu2,0)
      B0k1=qlI2(k1Dk1,msq,msq,mu2,0)
      B0k2=qlI2(k2Dk2,msq,msq,mu2,0)
      B0k12=qlI2(k12Dk12,msq,msq,mu2,0)
! catch results that need to be computed in quad precision (?!)
      if (B0k1 /= B0k1) then
        msq_128 = real(msq,kind=real128); mu2_128=real(mu2,kind=real128)
        B0k1=qlI2q(real(k1Dk1,kind=real128),msq_128,msq_128,mu2_128,0)
      endif
      if (B0k2 /= B0k2) then
        msq_128 = real(msq,kind=real128); mu2_128=real(mu2,kind=real128)
        B0k2=qlI2q(real(k2Dk2,kind=real128),msq_128,msq_128,mu2_128,0)
      endif
      if (B0k12 /= B0k12) then
        msq_128 = real(msq,kind=real128); mu2_128=real(mu2,kind=real128)
        B0k12=qlI2q(real(k12Dk12,kind=real128),msq_128,msq_128,mu2_128,0)
      endif
      Gram=k1Dk1*k2Dk2-k1Dk2**2
      FL=-((2._dp-3._dp*k1Dk1*k2Dk12/Gram)*(B0k1-B0k12)
     &    +(2._dp-3._dp*k2Dk2*k1Dk12/Gram)*(B0k2-B0k12)
     &  -(4._dp*mtsq+k1Dk1+k2Dk2+k12Dk12-3._dp*k1Dk1*k2Dk2*k12Dk12/Gram)
     &  *C0k1xk2-2._dp)/Gram;
      FT=-(k12Dk12*(B0k1+B0k2-2._dp*B0k12-2._dp*k1Dk2*C0k1xk2)
     & +(k1Dk1-k2Dk2)*(B0k1-B0k2))/Gram-k1Dk2*FL;
      return
      end
