!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function pmpmC12x34(p1,p2,p3,p4,za,zb,msqCoeff)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
!     Implementation of Eq.~(6.11) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp)::pmpmC12x34,scpmpmC12x34m0,pmpmC12x34m2,
     & pmpmC12x34m0diff,msqCoeff

      msqCoeff=pmpmC12x34m2(p1,p2,p3,p4,za,zb)
      pmpmC12x34= +scpmpmC12x34m0(p1,p2,p3,p4,za,zb)
     &            +pmpmC12x34m0diff(p1,p2,p3,p4,za,zb)
     &            +pmpmC12x34m0diff(p3,p4,p1,p2,za,zb)
     &            +pmpmC12x34m0diff(p2,p1,p4,p3,zb,za)
     &            +pmpmC12x34m0diff(p4,p3,p2,p1,zb,za)
     &            +mtsq*msqCoeff
      return
      end

      function pmpmC12x34m0diff(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: pmpmC12x34m0diff,zab2

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      
      pmpmC12x34m0diff=
     & ((za(p1,p2)*zb(p1,p3)*za(p3,p4))**2
     & -za(p2,p4)**2*zab2(p1,p3,p4,p1)*zab2(p4,p1,p2,p4))
     & /(za(p1,p2)*za(p3,p4)*zab2(p1,p3,p4,p2)*zab2(p3,p1,p2,p4))

      return
      end

