!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function pmpmC12x34m2(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: pmpmC12x34m2,pmpmC12x34m2part
      pmpmC12x34m2=
     & +pmpmC12x34m2part(p1,p2,p3,p4,za,zb)
     & +pmpmC12x34m2part(p3,p4,p1,p2,za,zb)
     & +pmpmC12x34m2part(p2,p1,p4,p3,zb,za)
     & +pmpmC12x34m2part(p4,p3,p2,p1,zb,za)
      return
      end


      function pmpmC12x34m2part(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^- 3_g^+ 4_g^- +H
!     Implementation of Eq.~(6.12) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s12,s13,s14,s23,s24,s34,s1234,Delta
      complex(dp)::pmpmC12x34m2part,zab2,zab2341,zab1342,zab3124
 
      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)

      s12=s(p1,p2)
      s13=s(p1,p3)
      s14=s(p1,p4)
      s23=s(p2,p3)
      s24=s(p2,p4)
      s34=s(p3,p4)
      s1234=s12+s13+s14+s23+s24+s34
      Delta=(s1234-s12-s34)**2-4._dp*s12*s34
      zab2341=zab2(p2,p3,p4,p1)
      zab1342=zab2(p1,p3,p4,p2)
      zab3124=zab2(p3,p1,p2,p4)
      pmpmC12x34m2part=
     & 4._dp*zab2341/(zab1342*zab3124)
     & *(za(p2,p4)*(s13+s23-s14-s24)/Delta
     & *(zb(p2,p3)-za(p1,p4)*(s1234-s12-s34)
     & /(2._dp*za(p1,p2)*za(p3,p4)))
     & +zb(p2,p3)**2*(s23-s14)/(zb(p1,p2)*zb(p3,p4)*zab1342)
     & +1.5_dp*zb(p1,p3)*zb(p2,p3)/(zb(p1,p2)*zb(p3,p4)))
      return
      end


