!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function ppmmB23(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^+ 3_g^- 4_g^- +H
!     Implementation of Eq.~(7.17) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: ppmmB23,ppmmB23symm

      ppmmB23=ppmmB23symm(p1,p2,p3,p4,za,zb)
     &       +ppmmB23symm(p4,p3,p2,p1,zb,za)

      return
      end


      function ppmmB23symm(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^+ 3_g^- 4_g^- +H
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/Inc/sprods_com.f'
      integer p1,p2,p3,p4
      real(dp):: s14,s23,s123,s124,s134,s234,s1234,Delta
      complex(dp):: ppmmB23symm,zab2

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      
      s14=s(p1,p4)
      s23=s(p2,p3)
      s123=s(p1,p2)+s(p1,p3)+s(p2,p3)
      s124=s(p1,p2)+s(p1,p4)+s(p2,p4)
      s134=s(p1,p3)+s(p1,p4)+s(p3,p4)
      s234=s(p2,p3)+s(p2,p4)+s(p3,p4)
      s1234=s(p1,p2)+s(p1,p3)+s(p1,p4)+s(p2,p3)+s(p2,p4)+s(p3,p4)
      
      Delta=(s1234-s23-s14)**2-4._dp*s23*s14

      ppmmB23symm = 
     & -4._dp*zb(p2,p4)**2*za(p3,p4)*s234
     & /(zb(p3,p4)*zab2(p4,p2,p3,p4)*zab2(p1,p2,p3,p4)**2)
     & -4._dp*zb(p1,p3)**2*za(p2,p3)*zab2(p1,p2,p3,p1)*zab2(p3,p4,p1,p2)
     & *(2._dp*s124+s134-3._dp*s14)
     &  /(zb(p1,p4)*zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)**2*Delta)
     &  +12._dp*za(p1,p4)*zb(p1,p3)**2*za(p2,p3)**2
     & *zb(p2,p3)*zab2(p3,p4,p1,p2)
     &  /(zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)**2*Delta)
     & +4._dp*za(p2,p4)*zb(p2,p4)*zab2(p3,p4,p1,p2)
     & /(zab2(p1,p2,p3,p4)**2*zab2(p2,p1,p4,p3))
     & -8._dp*zb(p2,p4)*za(p2,p3)*zab2(p3,p4,p1,p2)*(
     &     za(p1,p4)*zb(p3,p4)*zab2(p4,p2,p3,p1)
     &    +za(p1,p4)*za(p2,p3)*zb(p1,p3)*zb(p2,p3)
     &    +za(p2,p4)*zb(p2,p3)*zab2(p4,p2,p3,p4))
     &  /(zab2(p1,p2,p3,p4)**2*zab2(p2,p1,p4,p3)*Delta)
     & -4._dp*za(p2,p4)*zb(p1,p3)*zab2(p3,p4,p1,p2)
     &  /(zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)**2)
     & +8._dp*za(p1,p4)*zb(p3,p4)*zb(p1,p3)*za(p2,p3)
     & *zab2(p4,p2,p3,p1)*zab2(p3,p4,p1,p2)
     &  /(zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)**2*Delta)
     &     +3._dp*zab2(p4,p2,p3,p1)*zab2(p3,p4,p1,p2)
     &  *(s124-s134)*(s123-s234)*(s234+s123)
     &  /(zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)*Delta**2)
     & +zab2(p4,p2,p3,p1)*zab2(p3,p4,p1,p2)
     & *(2._dp*s23+5._dp*s(p2,p4)+3._dp*s(p3,p4)
     &  +3._dp*s(p1,p2)+5._dp*s(p1,p3))
     &  /(zab2(p1,p2,p3,p4)*zab2(p2,p1,p4,p3)*Delta)

      return
      end

