!  Copyright (C) 2019-2021, respective authors of MCFM.
!
!  This program is free software: you can redistribute it and/or modify it under
!  the terms of the GNU General Public License as published by the Free Software
!  Foundation, either version 3 of the License, or (at your option) any later
!  version.
!
!  This program is distributed in the hope that it will be useful, but WITHOUT ANY
!  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
!  PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License along with
!  this program. If not, see <http://www.gnu.org/licenses/>
 
      function ppmmC23x41(p1,p2,p3,p4,za,zb,msqCoeff)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^+ 3_g^- 4_g^- +H
!     Implementation of Eq.~(7.12) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      include 'src/ggHgg_mass/mtsq.f'
      integer p1,p2,p3,p4
      complex(dp):: ppmmC23x41,scppmmC23x41m0,ppmmC23x41m2,
     & ppmmC23x41m0diff,msqCoeff

      msqCoeff=ppmmC23x41m2(p1,p2,p3,p4,za,zb)
      ppmmC23x41=+scppmmC23x41m0(p1,p2,p3,p4,za,zb)
     &           +ppmmC23x41m0diff(p1,p2,p3,p4,za,zb)
     &           +ppmmC23x41m0diff(p2,p1,p4,p3,za,zb)
     &           +ppmmC23x41m0diff(p3,p4,p1,p2,zb,za)
     &           +ppmmC23x41m0diff(p4,p3,p2,p1,zb,za)
     &           +mtsq*msqCoeff
     
      return
      end




      function ppmmC23x41m0diff(p1,p2,p3,p4,za,zb)
      implicit none
!     This is a calculation for the process 1_g^+ 2_g^+ 3_g^- 4_g^- +H
!     Implementation of Eq.~(7.12) from arXiv:2002.04018 v2
      include 'src/Inc/types.f'
      include 'src/Inc/mxpart.f'
      include 'src/Inc/zprods_decl.f'
      integer p1,p2,p3,p4
      complex(dp):: ppmmC23x41m0diff,zab2

      zab2(p1,p2,p3,p4)=za(p1,p2)*zb(p2,p4)+za(p1,p3)*zb(p3,p4)
      
      ppmmC23x41m0diff=
     & (za(p3,p4)**2*zab2(p2,p1,p4,p2)*zab2(p4,p2,p3,p4)
     & -(za(p2,p3)*zb(p2,p1)*za(p1,p4))**2)
     & /(za(p2,p3)*za(p1,p4)*zab2(p2,p1,p4,p3)*zab2(p1,p2,p3,p4))
      return
      end

