      subroutine fviqqx(fi,vc,cl,cr,fmass, key, fvi)
c
c This subroutine computes an off-shell fermion wavefunction from a
c flowing-IN external fermion and a vector boson.
c
c input:
c       complex fi(6)          : flow-in  fermion                   |fi>
c       complex vc(6)          : input    vector                      v
c       real    fmass          : mass  of output fermion f'
c
c output:
c       complex fvi(6)         : off-shell fermion             |f',v,fi>
c
      implicit none
      integer key
      double complex fi(6),vc(6),fvi(6),d
      double precision pf(0:3),fmass,pf2,cl,cr
      
      double precision rZero, rOne
      parameter( rZero = 0.0d0, rOne = 1.0d0 )
      double complex cImag, cZero
      parameter( cImag = ( 0.0d0, 1.0d0 ), cZero = ( 0.0d0, 0.0d0 ) )


      fvi(5) = fi(5)-vc(5)
      fvi(6) = fi(6)-vc(6)

      pf(0) = dble( fvi(5))
      pf(1) = dble( fvi(6))
      pf(2) = dimag(fvi(6))
      pf(3) = dimag(fvi(5))
      pf2 = pf(0)**2-(pf(1)**2+pf(2)**2+pf(3)**2)


      if(key.eq.1) then
      d = -rOne/dcmplx( pf2-fmass**2, 0.d0)
      elseif(key.eq.0) then
      d = -rOne
      else
              print *, "wrong key in helas!"
              stop
      endif

         fvi(1)=cr*fmass*fi(4)*(-vc(2) + cImag*vc(3)) + 
     -  cl*fi(2)*((pf(0) - pf(3))*(vc(2) - cImag*vc(3)) + 
     -     (-pf(1) + cImag*pf(2))*(vc(1) - vc(4))) + 
     -  cr*fmass*fi(3)*(vc(1) - vc(4)) + 
     -  cl*fi(1)*((-pf(1) + cImag*pf(2))*
     -      (vc(2) + cImag*vc(3)) + 
     -     (pf(0) - pf(3))*(vc(1) + vc(4)))

         fvi(2)=-(cr*fmass*fi(3)*(vc(2) + cImag*vc(3))) + 
     -  cl*fi(2)*((pf(1) + cImag*pf(2))*
     -      (-vc(2) + cImag*vc(3)) + 
     -     (pf(0) + pf(3))*(vc(1) - vc(4))) + 
     -  cr*fmass*fi(4)*(vc(1) + vc(4)) + 
     -  cl*fi(1)*((pf(0) + pf(3))*(vc(2) + cImag*vc(3)) - 
     -     (pf(1) + cImag*pf(2))*(vc(1) + vc(4)))

         fvi(3)=cl*fmass*fi(2)*(vc(2) - cImag*vc(3)) + 
     -  cr*fi(3)*(-((pf(1) - cImag*pf(2))*
     -        (vc(2) + cImag*vc(3))) + 
     -     (pf(0) + pf(3))*(vc(1) - vc(4))) + 
     -  cl*fmass*fi(1)*(vc(1) + vc(4)) + 
     -  cr*fi(4)*((pf(0) + pf(3))*
     -      (-vc(2) + cImag*vc(3)) + 
     -     (pf(1) - cImag*pf(2))*(vc(1) + vc(4)))

         fvi(4)=cl*fmass*fi(1)*(vc(2) + cImag*vc(3)) + 
     -  cr*fi(3)*(-((pf(0) - pf(3))*
     -        (vc(2) + cImag*vc(3))) + 
     -     (pf(1) + cImag*pf(2))*(vc(1) - vc(4))) + 
     -  cl*fmass*fi(2)*(vc(1) - vc(4)) +
     -  cr*fi(4)*((pf(1)+cImag*pf(2))*
     -     (-vc(2)+cImag*vc(3)) +
     -    (pf(0)-pf(3))*(vc(1)+vc(4))) 

         fvi(1)=d*fvi(1)
         fvi(2)=d*fvi(2)
         fvi(3)=d*fvi(3)
         fvi(4)=d*fvi(4)


      return
      end
