      subroutine jioqqx(fi,fo, cl,cr,vmass, key, jio)
c
c This subroutine computes an off-shell vector current from an external
c fermion pair.  The vector boson propagator is given in Feynman gauge
c for a massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex fi(6)          : flow-in  fermion                   |fi>
c       complex fo(6)          : flow-out fermion                   <fo|
c
c output:
c       complex jio(6)         : vector current          j^mu(<fo|v|fi>)
c     
      implicit none
      integer key
      double complex fi(6),fo(6),jio(6),d
      double precision q(0:3),q2,cl,cr,vmass(2)


      double precision rZero, rOne
      parameter( rZero = 0.0d0, rOne = 1.0d0 )
      double complex cImag, cZero
      parameter( cImag = ( 0.0d0, 1.0d0 ), cZero = ( 0.0d0, 0.0d0 ) )


      jio(5) = fo(5)-fi(5)
      jio(6) = fo(6)-fi(6)

      q(0) = dble( jio(5))
      q(1) = dble( jio(6))
      q(2) = dimag(jio(6))
      q(3) = dimag(jio(5))
      q2 = q(0)**2-(q(1)**2+q(2)**2+q(3)**2)


      if(key.eq.1) then
         d = rOne/dcmplx( q2-vmass(1)**2, vmass(1)*vmass(2))
      elseif(key.eq.0) then
         d= rOne
      else
              print *, "wrong key in helas!"
              stop
      endif

        jio(1)=cr*fi(3)*fo(1) + cr*fi(4)*fo(2) + 
     -  cl*fi(1)*fo(3) + cl*fi(2)*fo(4)

        jio(2)=cr*(fi(4)*fo(1) + fi(3)*fo(2)) - 
     -  cl*(fi(2)*fo(3) + fi(1)*fo(4))

        jio(3)=cImag*(-(cr*fi(4)*fo(1)) + cr*fi(3)*fo(2) + 
     -    cl*fi(2)*fo(3) - cl*fi(1)*fo(4))

        jio(4)=cr*fi(3)*fo(1) - cr*fi(4)*fo(2) - 
     -  cl*fi(1)*fo(3) + cl*fi(2)*fo(4)

         jio(1)=d*jio(1)
         jio(2)=d*jio(2)
         jio(3)=d*jio(3)
         jio(4)=d*jio(4)

      return
      end
